/* =========================================================================
   ================================ ASL ====================================
   =========================================================================
       ________________________________________________________________
      |Solve an optimization problem with linear and bound constraints |
      |                                                                |
      |  (BL) min f(x)  subject to  lo <= x <= hi,  a'x = b            |
      |                                                                |
      |  using the affine-scaling interior-point method presented in:  |
      |                                                                |
      |   An affine-scaling interior-point CBB method for continuous   |
      |   knapsack constraints by Maria D. Gonzalez-Lima, William W.   |
      |   Hager, and Hongchao Zhang.                                   |
      |                                                                |
      |                  Version 1.1 (July 25, 2009)                   |
      |                                                                |
      |       William W. Hager                 Hongchao Zhang          |
      |      hager@math.ufl.edu             hozhang@math.lsu.edu       |
      |   Department of Mathematics       Department of Mathematics    |
      |    University of Florida         Louisiana State University    |
      |  Gainesville, Florida 32611        Baton Rouge, LA 70803       |
      |      352-392-0281 x 244                 225-578-1982           |
      |http://www.math.ufl.edu/~hager     www.math.lsu.edu/~hozhang    |
      |                                                                |
      |      Copyright by William W. Hager and Hongchao Zhang          |
      |________________________________________________________________|

       ________________________________________________________________
      |This program is free software; you can redistribute it and/or   |
      |modify it under the terms of the GNU General Public License as  |
      |published by the Free Software Foundation; either version 2 of  |
      |the License, or (at your option) any later version.             |
      |This program is distributed in the hope that it will be useful, |
      |but WITHOUT ANY WARRANTY; without even the implied warranty of  |
      |MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the   |
      |GNU General Public License for more details.                    |
      |                                                                |
      |You should have received a copy of the GNU General Public       |
      |License along with this program; if not, write to the Free      |
      |Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, |
      |MA  02110-1301  USA                                             |
      |________________________________________________________________|

      NOTE: The code can exploit special structure:

            1. If the upper and lower bounds are scalars, not vectors,
               then set Uparm->bnd_is_scalar = 1. *lo and *hi each contain one
               entry, the scalar lower and upper bounds.
               Uparm->bnd_is_scalar = 0 by default.

            2. If a = 1 identically, then set Uparm->a_is_1 = 1 and put
               NULL for the a argument of ASL. Uparm->a_is_1 = 0 by default.

            3. If f(x) = .5x'Ax + c'x (quadratic objective function),
               set Uparm->QuadCost = 1. When the routine Ad below is
               invoked, it should return the product A*d in Objective->p.
               Uparm->QuadCost = 0 by default. For a quadratic objective,
               the user should provide the gradient at the starting x
               in ASL argument g0 (= Ax + c). The returned objective
               function value is the difference between the starting
               cost and the final (optimal) cost. If the user wants the
               code to compute g0 and the final cost (not the cost difference),
               then store c in the g0 argument and set Parm->QuadLazy = 1.

            4. For a general objective function, if the function value
               and gradient can be computed together at the same speed
               as the value alone, then replace the value argument of
               ASL by valgrad if valgrad is not NULL. */

/* Contents:

    1. ASL         - solve optimization problem with linear + bound constraints
    2. ASLevaluate - evaluate objective function or gradient
    3. ASLd        - evaluate the search direction
    4. ASLval      - evaluate r(C) = a'd(C) and r'(C)
    5. ASLbb       - compute lambda_k using BB formula
    6. ASLgather   - gather and compress elements of an array
    7. ASLscatter  - scatter elements of a compressed array
    8. ASLstep     - evaluate xnew = x + st*d
    9. ASLdot      - compute x dot y, x and y dense
    10  ASLdoti    - compute x dot y, x sparse and y dense
    11. ASLadd     - add components of a vector
    12. ASLcopy    - copy y to x
    13. ASLmax     - compute infinity norm of a vector
    14. ASLint_init- initialize an integer array
    15. ASLpartialMinSort - extract smallest elements of an array
    16. ASLpartialMaxSort - extract largest elements of an array
    17. ASLminHeapSort    - sort a min heap
    18. ASLmaxHeapSort    - sort a max heap
    19. ASLminheap_build  - build a min heap
    20. ASLminheap_delete - delete top element in a min heap
    21. ASLminheap_add    - add new leaf to a min heap
    22. ASLminheapify     - min heapify starting at given node
    23. ASLmaxheap_build  - build a max heap
    24. ASLmaxheap_delete - delete top element in a max heap
    25. ASLmaxheap_add    - add new leaf to a max heap
    26. ASLmaxheapify     - max heapify starting at given node
    27. ASLnapsack        - solve projection problem for napsack polytope
    28. ASLnapup          - used by napsack when multiplier guess too small
    29. ASLnapdown        - used by napsack when multiplier guess too big
    30. ASLdefault        - set default parameter values
    31. ASLprint_parms    - print the parameter array */

#include "ASL.h"

int ASL /* return:
                   0 (error tolerance satisfied)
                   1 (number of iterations exceeded limit)
                   2 (out of memory)
                   3 (line search failed)
                   4 (search direction not a descent direction)
                   5 (computation of search direction fails)
                   6 (search direction not a descent direction )
                   7 (computation of mu fails) 
                   8 (function value is nan in line search)*/
(
    ASLFLOAT      *x, /* input: starting guess, output: the solution */
    ASLFLOAT     *lo, /* lower bounds (single bound if bnd_is_scalar = 1) */
    ASLFLOAT     *hi, /* upper bounds (single bound if bnd_is_scalar = 1) */
    ASLFLOAT      *a, /* linear constraint vector in a'x = b
                         NULL = no linear constraint or a identically 1
                         if Uparm->a_is_1 = 1 */
    ASLFLOAT       b, /* right side */
    ASLFLOAT     *g0, /* if cost quadratic, g0 = Ax + c = starting gradient */
    int            n, /* problem dimension */
    ASLFLOAT grad_tol,/* bound on KKT error */
    /* any of the remaining arguments can be NULL */
    ASLstat    *Stat, /* structure with statistics (can be NULL) */
    ASLparm   *Uparm, /* user parameters, NULL = use default parameters */
    void      (*Ad) (ASLobjective *), /* compute A*d if quadratic objective */
    void   (*value) (ASLobjective *), /* evaluate objective function */
    void    (*grad) (ASLobjective *), /* evaluate objective gradient */
    void (*valgrad) (ASLobjective *)  /* evaluate function and gradient
                                         NULL = use value & grad routines */
)
{
    int bnd_is_scalar, fmemp, j, k, l, m, ia, M, max_backsteps, PrintLevel,
        status, QuadCost, mm, ll, nl, nn, np, ny, nz, n2, ntot, nmin,
        AArmijo, Subspace, a_is_1, nsub,
        *i, *iwork, *iw1, *iw2, *id, *idnn, *iy, *ikeep ;
    long N ;
    ASLFLOAT fmax, fr, fdelete, fnew, f, fmin, fc, alpha, err, eps, X, Y,
          lambda, lambda0, lambda_k, dum, atemp, normg, gtd, normx, r, s, t,
          mu, nu, Armijo_hi, AArmijo_hi, delta, armijo_decay, armijo0, armijo1,
          fcomp, fr_pert, fcomp_pert, dphia, lo_cut, hi_cut, Clo, Chi, xj,
         *xComp, *xnewComp, *gComp, *aComp, *loComp, *hiComp,
         *d, *dnn, *g, *xnew, *y, *gnew, *work, *fmem, *w1 ;
    ASLINT it ;
    ASLobjective Objective ;
    ASLparm *Parm, ParmStruc ;
    ASLcom Com ;

    /* ---------------------------------------------------------------------- */
    /* set parameter values */
    /* ---------------------------------------------------------------------- */
    if ( Uparm == NULL )
    {
        Parm = &ParmStruc ;
        ASLdefault (Parm) ;
    }
    else
    {
        Parm = Uparm ;
    }

    if ( Parm->PrintParms ) ASLprint_parms (Parm, grad_tol, n) ;
    lo_cut = Parm->lo_cut ;
    hi_cut = Parm->hi_cut ;
    QuadCost = Parm->QuadCost ;
    PrintLevel = Parm->PrintLevel ;
    lambda0 = Parm->lambda0 ;
    Subspace = Parm->Subspace ;
    bnd_is_scalar = Parm->bnd_is_scalar ;
    a_is_1 = Parm->a_is_1 ;
    if ( bnd_is_scalar ) { Clo = *lo ; Chi = *hi ; }

    Com.Parm = Parm ;
    Com.n = n ;
    N = (long) n ;
    Com.lo = lo ;
    Com.hi = hi ;
    Com.a = a ;
    Com.b = b ;
    /* compute max norm of linear constraint vector, gradient, and x */
    if ( Parm->a_is_1 ) t = 1 ;
    else
    {
        t = ASLZERO ;
        if ( a != NULL ) t = ASLmax (a, n) ;
    }
    Com.norma = t ;

    /* ---------------------------------------------------------------------- */
    /* allocate memory */
    /* ---------------------------------------------------------------------- */
    if ( Subspace )
    {
        /* d, g, xnew, gnew, w1, y, fmem,  xComp, xnewComp, gComp,
           aComp, loComp, hiComp */
        k = 12 ;
        if ( a_is_1 ) k-- ;
        j = 0 ;
        if ( bnd_is_scalar ) { k -= 2 ; j = 2 ; }
        work = (double *) malloc ((k*N + Parm->M + j)*sizeof (ASLFLOAT)) ;
        /* i, iw1, iw2, iy, id, ikeep */
        iwork = (int *) malloc ((6*N)*sizeof (int)) ;
        /* subspace size */
        nsub = Parm->nsub ;
        if ( nsub <= 0 )
        {
            nsub = (int) n*Parm->fsub ;
            nsub = ASLMAX (2, nsub) ;
        }
    }
    else
    {
        work = (double *) malloc ((4*N + Parm->M)*sizeof (ASLFLOAT)) ;
        iwork = (int *) malloc (N*sizeof (int)) ;
    }
    if ( (work == NULL) || (iwork == NULL) )
    {
        printf ("Insufficient memory for specified problem dimension %e\n",
                 (double) n) ;
        return (2) ;
    }
    d = work ;
    g = d+n ;
    xnew = g+n ;
    gnew = xnew+n ;
    fmem = gnew+n ;
    i = iwork ;
    if ( Subspace )
    {
        iw1 = i+n ;
        iw2 = iw1+n ;
        id = iw2+n ;
        iy = id+n ;
        ikeep = iy+n ;
        ASLint_init (ikeep, 0, n) ;
        w1 = fmem+(Parm->M) ;
        y = w1+n ;
        /* storage for compressed arrays */
        xComp = y+n ;
        xnewComp = xComp+n ;
        gComp = xnewComp+n ;
        if ( a_is_1 )
        {
            aComp = NULL ;
            loComp = gComp+n ;
            if ( bnd_is_scalar )
            {
                hiComp = loComp+1 ;
                *loComp = *lo ;
                *hiComp = *hi ;
            }
            else hiComp = loComp+n ;
        }
        else
        {
            aComp = gComp+n ;
            loComp = aComp+n ;
            if ( bnd_is_scalar )
            {
                hiComp = loComp+1 ;
                *loComp = *lo ;
                *hiComp = *hi ;
            }
            else hiComp = loComp+n ;
        }
    }

    /* Armijo line search parameters */
    delta = Parm->Armijo_delta ;
    armijo_decay = Parm->Armijo_decay ;
    max_backsteps = Parm->max_backsteps ;
    armijo0 = Parm->armijo0 ;
    armijo1 = Parm->armijo1 ;
    AArmijo = Parm->AArmijo ;

    /* starting function value and gradient */
    Objective.x = x ;
    Objective.g = g ;
    Objective.i = i ;
    Objective.n = n ;
    Objective.info = -1 ; /* initial evaluation */
    Objective.p = xnew ; /* we don't need xnew if quadratic, use space for p */
    Com.user = &Objective ;
    Com.Ad = Ad ;
    Com.value = value ;
    Com.grad = grad ;
    Com.valgrad = valgrad ;
    /* initialize counters for statistics */
    it = 0 ;
    Com.nf = 0 ;
    Com.ng = 0 ;
    Com.nmu = 0 ;

    if ( Parm->QuadCost )
    {
        if ( Parm->QuadLazy )
        {
            nz = 0 ;
            for (j = 0; j < n; j++)
            {
                if ( x [j] != ASLZERO )
                {
                    d [nz] = x [j] ;
                    i [nz] = j ;
                    nz++ ;
                }
            }
            Objective.nz = nz ;
            Ad (&Objective) ;
            Com.ng = 1 ;
            /* NOTE: g0 stores linear term c in quadratic cost, g = Ax + c */
            ASLstep (g, g0, Objective.p, ASLONE, n) ;
            /* f = .5(x'g + c'x) = .5(x'(Ax+c) + c'x) = .5x'Ax + c'x */
            Objective.f = .5*(ASLdot (x, g, n) + ASLdot (x, g0, n)) ;
        }
        else
        {
            ASLcopy (g, g0, n) ;
            Objective.f = ASLZERO ;
            Com.ng = 0 ;
        }
    }
    else ASLevaluate (&Com, "fg") ; /* evaluate function and gradient */
    f = Objective.f ;

    /* henceforth, function value = fnew and gradient = gnew */
    Objective.x = xnew ;
    ASLcopy (xnew, x, n) ;
    Objective.xold = x ;
    Objective.g = gnew ;
    Objective.gold = g ;
    Objective.d = d ;

    if ( PrintLevel >= 1 ) printf ("starting function value:  %e\n", f) ;

    /* initialize function value memory */
    M = Parm->M ;
    for (k = 1; k < M; k++) fmem [k] = -ASLINF ;
    fmem [0] = f ;
    fmemp = 1 ;    /* pointer to next available location in fmem */
    fmax = f ;
    err = ASLINF ;
    mm = 0 ; /* number of iterations in the current CBB cycle */
    ll = 0 ; /* zero as long as unit steps in line search */
    nl = 0 ; /* number of iterations since fmin decreased in value */
    np = 0 ; /* number of times initial stepsize was accepted in line search */
    fmin = f ;
    fr = f ;
    fc = f ;

    normg = ASLmax (g, n) ;
    normx = ASLmax (x, n) ;
    if ( PrintLevel >= 2 )
    {
        printf ("starting max norm of iterate: %e gradient:  %e\n",
                 normx, normg) ;
    }
    if ( normx != ASLZERO)  lambda_k = ASLMAX (normg/normx, lambda0) ;
    else                    lambda_k = ASLMAX (normg, lambda0) ;
    lambda_k = ASLMAX (lambda_k, Parm->lambda0Factor*normg) ;
    while ( it < Parm->maxits )
    {
        it++ ;

        if ( Subspace )
        {
         /* If Subspace = TRUE, then solve the problem:

                min .5*lambda_k ||d|| + g_k'd s.t. lo <= x + d <= hi, a'd = 0

            to determine which components of d yield biggest cost reduction.
            With the change of variables z = x + d, this is equivalent to

            (P) min ||z-y|| subject to lo <= z <= hi, a'z = b

            where y = x - g/lambda_k. Let mu denote the multiplier
            connected with the Lagrangian of the projection problem (P):

            L_P(z, mu) = .5||z-y|| + mu(a'z - b)

            Let nu be the multiplier connected with the Lagrangian of (BL):

            L_B (x, nu) = f(x) + nu(a'x - b)

            As x converges a solution of (BL), the solution multipliers mu
            and nu have the following relation: 

            nu = lambda_k*mu */

            ASLstep (y, x, g, -ASLONE/lambda_k, n) ;
            if ( it == 1 ) /* estimate nu by assuming all components of z free*/
            {
                if ( a_is_1 )
                {
                    mu = (ASLadd (y, n) - b)/((ASLFLOAT) n) ;
                }
                else if ( a != NULL )
                {
                    mu = (ASLdot (a, y, n) - b)/ASLdot (a, a, n) ;
                }
            }
            if ( a_is_1 || (a != NULL) )
            {
                mu = ASLnapsack (y, mu, w1, iw1, iw2, &Com) ;
                nu = lambda_k*mu ;
            }
            else /* no equality constraint */
            {
                if ( bnd_is_scalar )
                {
                    for (j = 0; j < n; j++)
                    {
                        t = y [j] ;
                        if      ( t < Clo ) y [j] = Clo ;
                        else if ( t > Chi ) y [j] = Chi ;
                    }
                }
                else
                {
                    for (j = 0; j < n; j++)
                    {
                        t = y [j] ;
                        if      ( t < lo [j] ) y [j] = lo [j] ;
                        else if ( t > hi [j] ) y [j] = hi [j] ;
                    }
                }
                nu = ASLZERO ;
            }

            /* y now stores z, compute the following:
               1. error estimate based on the multiplier nu (smallest change
                  in g_j or xj for which optimality conditions hold)
               2. d = z - x */

            err = ASLZERO ;
            s = .5*lambda_k ;
            nz = 0 ;
            np = 0 ;
            nn = n ;
            eps = .001*grad_tol*ASLMIN (ASLONE/lambda_k, ASLONE) ;
            if ( a_is_1 || (a == NULL))
            {
                if ( bnd_is_scalar )
                {
                    for (j = 0; j < n; j++)
                    {
                        xj = x [j] ;
                        t = y [j] - xj ;
                        r = g [j] + nu ;
                        if ( fabs (t) >= eps ) /* include in d */
                        {
                            y [nz] = t*(s*t + r) ;
                            if ( t > ASLZERO )
                            {
                                d [np] = t ; /* d = z - x */
                                id [np] = j ;
                                iy [nz] = np ;
                                np++ ;
                            }
                            else
                            {
                                nn-- ;
                                d [nn] = t ;
                                id [nn] = j ;
                                iy [nz] = nn ;
                            }
                            nz++ ;
                        }
                        if ( r > ASLZERO )
                        {
                            X = xj - Clo ;
                            if ( r > X ) err = ASLMAX (err, X) ;
                            else         err = ASLMAX (err, r) ;
                        }
                        else
                        {
                            Y = Chi - xj ;
                            if (-r > Y ) err = ASLMAX (err, Y) ;
                            else         err = ASLMAX (err,-r) ;
                        }
                    }
                }
                else
                {
                    for (j = 0; j < n; j++)
                    {
                        xj = x [j] ;
                        t = y [j] - xj ;
                        r = g [j] + nu ;
                        if ( fabs (t) >= eps ) /* include in d */
                        {
                            y [nz] = t*(s*t + r) ;
                            if ( t > ASLZERO )
                            {
                                d [np] = t ; /* d = z - x */
                                id [np] = j ;
                                iy [nz] = np ;
                                np++ ;
                            }
                            else
                            {
                                nn-- ;
                                d [nn] = t ;
                                id [nn] = j ;
                                iy [nz] = nn ;
                            }
                            nz++ ;
                        }
                        if ( r > ASLZERO )
                        {
                            X = xj - lo [j] ;
                            if ( r > X ) err = ASLMAX (err, X) ;
                            else         err = ASLMAX (err, r) ;
                        }
                        else
                        {
                            Y = hi [j] - xj ;
                            if (-r > Y ) err = ASLMAX (err, Y) ;
                            else         err = ASLMAX (err,-r) ;
                        }
                    }
                }
            }
            else
            {
                if ( bnd_is_scalar )
                {
                    for (j = 0; j < n; j++)
                    {
                        xj = x [j] ;
                        t = y [j] - xj ;
                        r = g [j] + nu*a [j] ;
                        if ( fabs (t) >= eps ) /* include in d */
                        {
                            y [nz] = t*(s*t + r) ;
                            t *= a [j] ;
                            if ( t > ASLZERO )
                            {
                                d [np] = t ; /* d = z - x */
                                id [np] = j ;
                                iy [nz] = np ;
                                np++ ;
                            }
                            else
                            {
                                nn-- ;
                                d [nn] = t ;
                                id [nn] = j ;
                                iy [nz] = nn ;
                            }
                            nz++ ;
                        }
                        if ( r > ASLZERO )
                        {
                            X = xj - Clo ;
                            if ( r > X ) err = ASLMAX (err, X) ;
                            else         err = ASLMAX (err, r) ;
                        }
                        else
                        {
                            Y = Chi - xj ;
                            if (-r > Y ) err = ASLMAX (err, Y) ;
                            else         err = ASLMAX (err,-r) ;
                        }
                    }
                }
                else
                {
                    for (j = 0; j < n; j++)
                    {
                        xj = x [j] ;
                        t = y [j] - xj ;
                        r = g [j] + nu*a [j] ;
                        if ( fabs (t) >= eps ) /* include in d */
                        {
                            y [nz] = t*(s*t + r) ;
                            t *= a [j] ;
                            if ( t > ASLZERO )
                            {
                                d [np] = t ; /* d = z - x */
                                id [np] = j ;
                                iy [nz] = np ;
                                np++ ;
                            }
                            else
                            {
                                nn-- ;
                                d [nn] = t ;
                                id [nn] = j ;
                                iy [nz] = nn ;
                            }
                            nz++ ;
                        }
                        if ( r > ASLZERO )
                        {
                            X = xj - lo [j] ;
                            if ( r > X ) err = ASLMAX (err, X) ;
                            else         err = ASLMAX (err, r) ;
                        }
                        else
                        {
                            Y = hi [j] - xj ;
                            if (-r > Y ) err = ASLMAX (err, Y) ;
                            else         err = ASLMAX (err,-r) ;
                        }
                    }
                }
            }
            if ( err <= grad_tol )
            {
                status = 0 ;
                break ;
            }
            if ( nsub >= nz )
            {
                for (k = 0; k < nz; k++) i [k] = id [iy [k]] ;
                ntot = nz ;
                goto Compute_d ;
            }
            /* find nsub smallest elements of y */
            ASLpartialMinSort (i, y, nsub, nz) ; /* 0 <= i <= nz */

            /* find smallest element y_s of y and further compress i based
               on the ratio y_i/y_s */
            t = ASLZERO ;
            for (k = 0; k < nsub; k++)
            {
                s = y [i [k]] ;
                if ( s < t ) t = s ;
            }
            t *= Parm->big_yi ; /* if y > t, skip it */
            ny = 0 ;
            s = ASLZERO ;
            for (k = 0; k < nsub; k++)
            {
                l = i [k] ;
                if ( y [l] <= t )
                {
                    m = iy [l] ; /* iy in [0, np-1] or [nn, n-1] */
                    s += d [m] ;
                    j = id [m] ; /* 0 <= id <= n-1 */
                    i [ny] = j ;
                    ikeep [j] = 1 ;
                    ny++ ;
                }
            }
            /* i stores ny components of y with most impact on cost */

            ntot = ny ;
            n2 = nsub ;
            if ( s < ASLZERO )
            {
                nmin = ASLMIN (np, n2) ;
                /* find nmin largest elements of d */
                ASLpartialMaxSort (iw1, d, nmin, np) ;
                /* arrange indices for decreasing order in d */
                ASLminHeapSort (iw2, iw1, d, nmin) ;
                for (k = 0; k < nmin; k++)
                {
                    l = iw2 [k] ;
                    j = id [l] ;
                    if ( ikeep [j] ) continue ;
                    /* add components that help with constraint violation */
                    i [ntot] = j ;
                    ntot++ ;
                    s += d [l] ;
                    if ( s >= ASLZERO ) break ;
                }
            }
            else if ( s > ASLZERO )
            {
                nmin = ASLMIN (n-nn, n2) ;
                dnn = d+nn ;
                /* find nmin smallest elements of d */
                ASLpartialMinSort (iw1, dnn, nmin, n-nn) ;
                /* arrange indices for increasing order */
                ASLmaxHeapSort (iw2, iw1, dnn, nmin) ;
                idnn = id+nn ;
                for (k = 0; k < nmin; k++)
                {
                    l = iw2 [k] ;
                    j = idnn [l] ;
                    if ( ikeep [j] ) continue ;
                    /* add components that help with constraint violation */
                    i [ntot] = j ;
                    ntot++ ;
                    s += dnn [l] ;
                    if ( s <= ASLZERO ) break ;
                }
            }

            /* remove the 1s from ikeep */
            for (k = 0; k < ny; k++) ikeep [i [k]] = 0 ;
            Compute_d:
            ASLgather (xComp, x, i, ntot) ;/*gather subspace elements to xComp*/
            ASLgather (gComp, g, i, ntot) ;/*gather subspace elements to gComp*/
            if ( !a_is_1 ) ASLgather (aComp, a, i, ntot) ;
            if ( !bnd_is_scalar )
            {
                ASLgather (loComp, lo, i, ntot) ;
                ASLgather (hiComp, hi, i, ntot) ;
            }
            /* compute new search direction d, xnew = x + d, and gtd = g'd */
            if ( PrintLevel >= 1 )
            {
                printf ("subspace size before computing d: %i\n", ntot) ;
            }
            if ( it == 1 ) lambda = -nu ;

            status = ASLd (d, &lambda, xnewComp, &gtd, &dum, xComp, gComp,
                     aComp, loComp, hiComp, ntot, lambda_k, &Com);
            if ( !QuadCost ) ASLscatter (xnew, xnewComp, i, ntot) ;
        }
        else
        {
            if ( it == 1 ) /* estimate nu by assuming all components of z free*/
            {
                ASLstep (d, x, g, -ASLONE/lambda_k, n) ;
                if ( a_is_1 )
                {
                    lambda = -lambda_k*(ASLadd (d, n) - b)/((ASLFLOAT) n) ;
                }
                else if ( a != NULL )
                {
                    lambda = -lambda_k*(ASLdot (a, d, n) - b)/ASLdot (a, a, n) ;
                }
            }
            /* compute new search direction d, xnew = x + d, and gtd = g'd */
            status = ASLd (d, &lambda, xnew, &gtd, &err, x, g, a, lo, hi, n,
                           lambda_k, &Com) ;
        }
        nz = Objective.nz ;
        if ( status ) goto Exit ;

        /* check for convergence */
        if ( PrintLevel >= 1 )
        {
            printf ("nonzeros in d: %i\n", nz) ;
            printf ("iter: %i f: %e err: %e\n", it, f, err) ;
        }
        if ( err < grad_tol ) break ;

        if ( gtd > ASLZERO ) /* search direction not a descent direction */
        {
            status = 6 ;
            break ;
        }

        /* start line search */
        alpha = ASLONE ;
        ia = 0 ;
        Objective.info = 0 ;
        Objective.fold = f ;
        Objective.deriv1 = gtd ;
        Objective.step = alpha ;
        ASLevaluate (&Com, "f") ;
        fnew = Objective.f ;
        Objective.info = 1 ;
        if ( PrintLevel >= 2 )
            printf ("iter: %i iarmijo: %i f: %15.6e\n", (int) it, ia, fnew) ;

        if ( nl == Parm->L )
        {
            fr = fmax ;
            t = (fr-fmin)/(fc-fmin) ;
            if ( t > Parm->gamma1 ) fr = fc ;
            nl = 0 ;
        }

        if ( (np > Parm->P) && (fmax > f) )
        {
           t = (fr-f)/(fmax-f) ;
           if ( t > Parm->gamma2 ) fr = fmax ;
        }

        /* if initial step is not a CBB step, use nonmonotone line search */ 
        /*  fcomp = MIN (fmax, fr) ;  */
        fcomp = f;

        /* simplified line search when objective function is quadratic */
        if ( QuadCost )
        {
            if ( AArmijo)
            {
                if ( Parm->PertRule ) t = Parm->eps*fabs(fcomp) ;
                else                  t = Parm->eps ;
                fr_pert = fr + t ;
                fr = fr_pert ;
                fcomp_pert = fcomp + t ;
                if ( Parm->PrintLevel >= 1 )
                {
                    printf ("Perform approximate Armijo line search\n") ;
                    if ( Parm->PrintLevel >= 2 )
                    {
                        printf ("fr_pert: %14.6e fcomp_pert: %14.6e\n",
                                 fr_pert, fcomp_pert) ;
                    }
                }
                AArmijo_hi = (ASLTWO*delta - ASLONE)*gtd ;
                dphia = Objective.deriv1 + Com.deriv2 ;
                if ( mm == 0 )
                {
                    if ( fnew <= fr_pert )
                    {
                        if (dphia <= ASLTWO*(fr_pert - f) + AArmijo_hi )
                        {
                            mm++ ;
                            goto exit_cbbls ;
                        }
                    }
                }
                else
                {
                    if ( fnew <= fcomp_pert )
                    {
                        if (dphia <= ASLTWO*(fcomp_pert - f) + AArmijo_hi )
                        {
                            mm++ ;
                            goto exit_cbbls ;
                        }
                    }
                }
            }
            else
            {
                if ( PrintLevel >= 1 )
                {
                    printf ("Perform ordinary Armijo line search\n") ;
                }
                Armijo_hi = delta*gtd ;
                if ( mm == 0 ) t = fr ;
                else           t = fcomp ;
                if ( fnew <= t+Armijo_hi )
                {
                    mm++ ;
                    goto exit_cbbls ;
                }
            }
            /* if unit step failed, compute exact minimizer */
            if ( Com.deriv2 > ASLZERO )
            {
                ia = 1 ;
                alpha = -gtd/Com.deriv2 ;
                alpha = ASLMIN (alpha, ASLONE) ;
                fnew = f + alpha*(gtd + .5*alpha*Com.deriv2) ;
            }
            goto exit_cbbls ;
        }

        /* Approximate nonmonotone Armijo line search, decrease alpha until:
           phi'(alpha) <= [2(phi_r - phi(0))/alpha] + (2 delta - 1) phi'(0) and
           phi(alpha) <= phi_r, where phi_r = fr_pert or fcomp_pert. */
        if ( AArmijo)
        {
            if ( Parm->PertRule ) t = Parm->eps*fabs(fcomp) ;
            else                  t = Parm->eps ;
            fr_pert = fr + t ;
            fr = fr_pert ;
            fcomp_pert = fcomp + t ;
            if ( Parm->PrintLevel >= 1 )
            {
                printf ("Perform approximate Armijo line search\n") ;
                if ( Parm->PrintLevel >= 2 )
                {
                    printf ("fr_pert: %14.6e fcomp_pert: %14.6e\n",
                             fr_pert, fcomp_pert) ;
                }
            }

            AArmijo_hi = (ASLTWO*delta - ASLONE)*gtd ;
            if ( fnew != fnew ) /* function value is nan, reduce stepsize */
            {
                for (ia = 0; ia < Parm->nshrink; ia++)
                {
                    alpha *= Parm->nan_fac ;
                    ASLstepi (xnew, i, x, d, alpha, nz) ;
                    Objective.step = alpha ;
                    ASLevaluate (&Com, "f") ;
                    fnew = Objective.f ;
                    if ( fnew == fnew ) break ;
                }
                if ( (ia == Parm->nshrink) || (alpha == ASLZERO) )
                {
                    status = 8 ;
                    goto Exit ;
                }

                if ( fnew <= fcomp_pert)
                {
                    /* compute gradient gnew if not computed at xnew */
                    if ( value != valgrad ) ASLevaluate (&Com, "g") ;
                    dphia = ASLdot (gnew, d, n) ;
                    if (dphia <= ASLTWO*(fcomp_pert - f)/alpha + AArmijo_hi )
                        goto exit_cbbls ; /* unit step is valid */
                }
            }
            else
            {
                if ( mm == 0 )
                {
                    if ( fnew <= fr_pert )
                    {
                        /* compute gradient gnew if not computed at xnew */
                        if ( value != valgrad ) ASLevaluate (&Com, "g") ;
                        dphia = ASLdot (gnew, d, n) ;
                        if (dphia <= ASLTWO*(fr_pert - f) + AArmijo_hi )
                        {
                            mm++ ;
                            goto exit_cbbls ;
                        }
                    }
                }
                else
                {
                    if ( fnew <= fcomp_pert )
                    {
                        /* compute gradient gnew if not computed at xnew */
                        if ( value != valgrad ) ASLevaluate (&Com, "g") ;
                        dphia = ASLdot (gnew, d, n) ;
                        if (dphia <= ASLTWO*(fcomp_pert - f) + AArmijo_hi )
                        {
                            mm++ ;
                            goto exit_cbbls ;
                        }
                    }
                }
            }

            while ( ASLTRUE )
            {
                ia++ ;
                /* Modified Raydan's quadratic interpolation line search */
                t = ASLTWO*(fnew-f-alpha*gtd) ;
                if ( t != ASLZERO )
                {
                    atemp = (-gtd*alpha*alpha)/t ;
                    if ( (atemp < armijo0*alpha) || (atemp > armijo1*alpha ) )
                    {
                        atemp = armijo_decay*alpha ;
                    }
                    alpha = atemp ;
                }
                else alpha *= armijo_decay ;

                ASLstepi (xnew, i, x, d, alpha, nz) ;
                Objective.step = alpha ;
                ASLevaluate (&Com, "f") ;
                fnew = Objective.f ;
                if ( PrintLevel >= 2 )
                {
                     printf ("iter: %i iarmijo: %i f: %15.6e\n",
                              (int) it, ia, fnew) ;
                }

                if ( fnew <= fcomp_pert )
                {
                    /* compute gradient gnew if not computed at xnew */
                    if ( value != valgrad ) ASLevaluate (&Com, "g") ;
                    dphia = ASLdot (gnew, d, n) ;
                    if (dphia <= ASLTWO*(fcomp_pert - f)/alpha + AArmijo_hi )
                        goto exit_cbbls ;
                }

                if ( (alpha <= ASLZERO) || (ia >= Parm->max_backsteps) )
                {
                   status = 3 ;
                   goto Exit ;
                }
            }
            /* End of approximate Armijo line search */
        }
        /* Ordinary nonmonotone Armijo line search, decrease alpha until
           phi(alpha) <= phi_r + alpha * delta * phi'(0)
           where phi_r = fr or fcomp. */
        else
        {
            if ( PrintLevel >= 1 )
            {
                printf ("Perform ordinary Armijo line search\n") ;
            }
            Armijo_hi = delta*gtd ;
            if ( fnew != fnew ) /* function value is nan, reduce stepsize */
            {
                for (ia = 1; ia < Parm->nshrink; ia++)
                {
                    alpha *= Parm->nan_fac ;
                    ASLstepi (xnew, i, x, d, alpha, nz) ;
                    Objective.step = alpha ;
                    ASLevaluate (&Com, "f") ;
                    fnew = Objective.f ;
                    if ( fnew == fnew ) break ;
                }
                if ( (ia == Parm->nshrink) || (alpha == ASLZERO) )
                {
                    status = 8 ;
                    goto Exit ;
                }
                if ( fnew <= fcomp+alpha*Armijo_hi )
                {
                    if ( value != valgrad ) ASLevaluate (&Com, "g") ;
                    goto exit_cbbls ;
                }
            }
            else
            {
                if ( mm == 0 ) t = fr ;
                else           t = fcomp ;
                if ( fnew <= t+Armijo_hi )
                {
                    mm++ ;
                    if ( value != valgrad ) ASLevaluate (&Com, "g") ;
                    goto exit_cbbls ;
                }
            }

            while ( ASLTRUE )
            {
                ia++ ;
                /* Modified Raydan's quadratic interpolation line search */
                t = ASLTWO*(fnew-f-alpha*gtd) ;
                if ( t != ASLZERO )
                {
                    atemp = (-gtd*alpha*alpha)/t ;
                    if ( (atemp < armijo0*alpha) || (atemp > armijo1*alpha ) )
                    {
                        atemp = armijo_decay*alpha ;
                    }
                    alpha = atemp ;
                }
                else alpha *= armijo_decay ;

                Objective.step = alpha ;
                ASLstepi (xnew, i, x, d, alpha, nz) ;
                ASLevaluate (&Com, "f") ;
                fnew = Objective.f ;
                if ( PrintLevel >= 2 )
                {
                     printf ("iter: %i iarmijo: %i f: %15.6e\n",
                              (int) it, ia, fnew) ;
                }

                if ( fnew <= fcomp+alpha*Armijo_hi )
                {
                    if ( value != valgrad ) ASLevaluate (&Com, "g") ;
                    break ;
                }

                if ( (alpha <= ASLZERO) || (ia >= Parm->max_backsteps) )
                {
                    /* try approximate Armijo line search  */
                    if ( Parm->AArmijoFac > ASLZERO )
                    {
                       fr = fcomp ;
                       printf ("Armijo linesearch fails, "
                               "try Approximate Armijo linesearch") ;
                    }
                    else  /* line search fails */
                    {
                        status = 3 ;
                        goto Exit ;
                    }
                }
            } /* End of Armijo line search */
        }

        exit_cbbls:
        if ( fnew <= fmin )
        {
             fmin = fnew ;
             fc = fnew ;
             nl = 0 ;
        }
        else nl++ ;
        if ( fnew > fc ) fc = fnew ;

        ll += ia ; 
        if ( ia == 0 ) np++ ;
        else           np = 0 ;

        /* end of cbbls */

        if ( status ) goto Exit ; 

        /* if conditions hold, update BB step */
        if ( (ll >= 1) || (mm >= Parm->nm) || (it == 1) )
        {
            ll = 0 ;
            t = ASLbb (alpha, mm, &Com) ; 
            /* if t = -1, lambda_k does not change */ 
            if  ( t != -ASLONE ) 
            {
                lambda_k = t ;
                mm = 0 ;
            } 
        }
        else /* set x = xnew, g = gnew */
        {
            /* update x and g */
            if ( QuadCost )
            {
                for (k = 0; k < nz; k++) x [i [k]] += alpha*d [k] ;
                ASLstep (g, g, Objective.p, alpha, n) ;
            }
            else
            {
                for (k = 0; k < nz; k++)
                {
                    j = i [k] ;
                    x [j] = xnew [j] ;
                }
                ASLcopy (g, gnew, n) ;
            }
        }

        f = fnew ;

        /* update fmax */
        fdelete = fmem [fmemp] ;
        fmem [fmemp] = fnew ;
        fmemp++ ;
        if ( fmemp == M ) fmemp = 0 ; /* memory pointer loops from M to 0 */
        if ( fmax == fdelete ) /* find the max over all the memory */
        {
             fmax = -ASLINF ;
             for (k = 0; k < M; k++) if ( fmax < fmem [k] ) fmax = fmem [k] ;
        }
        else if ( fmax < fnew ) fmax = fnew ;

        if ( !AArmijo ) 
        {
            if ( fabs(fr - f) < Parm->AArmijoFac*fabs(fcomp) )
                AArmijo = ASLTRUE ;
        }
    }
    if ( it == Parm->maxits ) status = 1 ; /* iteration limit exceeded */

Exit:
    if ( f > fmin) f = fmin ;
    if ( err < grad_tol )     status = 0 ; /* termination criterion satisfied */

    if ( (Parm->PrintFinal) || (PrintLevel >= 1) )
    {
        if ( status == 0 )
        {
            printf ("Convergence tolerance for gradient satisfied\n") ;
        }
        else if ( status == 1 )
        {
            printf ("Number of iterations exceed specified limit\n") ;
            printf ("Iterations: %i maxit: %i\n", (int) it, Parm->maxits) ;
        }
        else if ( status == 2 )
        {
            printf ("Insufficient memory\n") ;
        }
        else if ( status == 3 )
        {
            printf ("Line search failed\n") ;
        }
        else if ( status == 4 )
        {
            printf ("Search direction not a descent direction\n") ;
        }
        else if ( status == 5 )
        {
            printf ("Computation of search direction fails\n") ;
        }
        else if ( status == 6 )
        {
            printf ("Search direction not a descent direction (gtd = %e)\n",
                     gtd) ;
        }
        else if ( status == 7 )
        {
            printf ("Number of mu iterations exceeds limit %i\n",
                     Parm->max_mu_its) ;
        }
        else if ( status == 8 )
        {
            printf ("Function value is nan in line search\n") ;
        }

        printf ("KKT error:         %13.6e\n", err) ;
        printf ("function value:    %13.6e\n", f) ;
        printf ("\nIterations:           %i\n", it) ;;
        printf ("Function evaluations: %i\n", (int) Com.nf) ;
        printf ("Gradient evaluations: %i\n", (int) Com.ng) ;
        printf ("Mu computation:       %i\n", (int) Com.nmu) ;
    }

    free (work) ;
    free (iwork) ;
    if ( Stat != NULL )
    {
        Stat->f = f ;
        Stat->err = err ;
        Stat->it = it ;
        Stat->nf = Com.nf ;
        Stat->ng = Com.ng ;
        Stat->nmu = Com.nmu ;
    }
    return (status) ;
}


/* ==========================================================================
   === ASLevaluate ==========================================================
   ==========================================================================
   Evaluate function, gradient, or both
   ========================================================================== */
PRIVATE void ASLevaluate
(
    ASLcom *Com,  /* ASLcom structure */
    char  *what   /* f = function, g = gradient, fg = function and gradient */
)
{
    ASLobjective *user ;
    user = Com->user ;
    /* Quadratic cost and initial evaluation */
    if ( !strcmp (what, "fg") )     /* compute function and gradient */
    {
        Com->ng++ ;
        Com->nf++ ;
        user->nz = user->n ;
        if ( Com->valgrad != NULL ) Com->valgrad (user) ;
        else
        {
            Com->value (user) ;
            Com->grad (user) ;
        }
    }
    else if ( !strcmp (what, "f") ) /* compute function */
    {
        if ( Com->Parm->QuadCost )
        {
            Com->ng++ ;
            Com->Ad (user) ;
            if ( user->nz == user->n )
                 Com->deriv2 = ASLdot  (user->d, user->p, user->n) ; /* d'Ad */
            else Com->deriv2 = ASLdoti (user->d, user->i, user->p, user->nz) ;
            user->f = user->fold + user->deriv1 + .5*Com->deriv2 ;
/*printf ("fold: %e\n", user->fold);
printf ("deriv1: %e\n", user->deriv1);
printf ("deriv2: %e\n", Com->deriv2);
printf ("alpha: %e\n", user->step) ;*/
        }
        else
        {
            int j, k, nz, *i ;
            ASLFLOAT step, *d, *x, *xold ;
            step = user->step ;
            if ( step != ASLONE )
            {
                i = user->i ;
                nz = user->nz ;
                d = user->d ;
                x = user->x ;
                xold = user->xold ;
                for (j = 0; j < nz; j++)
                {
                    k = i [j] ;
                    x [k] = xold [k] + step*d [k] ;
                }
            }
            Com->nf++ ;
            Com->value (user) ;
            if ( Com->valgrad == Com->value ) Com->ng++ ;
        }
    }
    else                                 /* compute gradient */
    {
        Com->ng++ ;
        Com->grad (user) ;
    }
}

/* ==========================================================================
   === ASLd =================================================================
   ==========================================================================
   Evaluate search direction d(mu) with the property that a'd(mu) = 0 where

                      g_j - mu a_j
   d_j (mu) = - ----------------------------
                 lambda_k + |g_j - mu a_j|/X

   X = hi_j - x_j if g_j - mu a_j <= 0
   X = x_j - lo_j if g_j - mu a_j  > 0

   If a is NULL, take mu = 0 to evaluate d.
   We first find an interval [a, b] containing mu:

           A = min_j g_j/a_j
           B = max_j g_j/a_j

    A trial mu is obtained by treating the denominator as constant and
    solving the linear equation a'd(mu) = 0 for mu = a'g/a'a.
    If mu is outside the interval [A, B], then we replace mu by the
    interval midpoint. Next we alternate between a Newton step and
    a secant step. If the pair of steps do not reduce the interval
    width by gamma*(B-A), then we perform a bisection step.

                                   a_j^2 lambda
    Note that a_j d_j'(mu) = --------------------------- .
                              (lambda - |g - mu a|/X)^2

    In the code which follows, the sign of d_j has been reversed.
   ========================================================================== */
PRIVATE int ASLd
(
    ASLFLOAT       *d, /* search dirction */
    ASLFLOAT      *mu, /* input guess for mu, return final mu */
    ASLFLOAT    *xnew, /* xnew = x + d */
    ASLFLOAT     *Gtd, /* g'd */
    ASLFLOAT     *Err, /* KKT error */
    ASLFLOAT       *x, /* current iterate */
    ASLFLOAT       *g, /* gradient at x */
    ASLFLOAT       *a, /* a'x = b */
    ASLFLOAT      *lo, /* lower bounds */
    ASLFLOAT      *hi, /* upper bounds */
    int             n, /* dimension */
    ASLFLOAT lambda_k, /* constant in denominator of d */
    ASLcom       *Com  /* common variables */
)
{
    int iu, j, nz, PrintLevel, method, max_mu_its, prior_side,
        a_is_1, bnd_is_scalar, QuadCost, Subspace, ok, *i ;
    ASLFLOAT A, B, C, D, gtd, err, r, t, u, alk, aj, da, db, dc, dd,
             va, vb, vc, vd, dj, gj, xj, X, Y, Clo, Chi, gamma, normg,
             epsmu0, epsmu1, tol0, tol1, width, lo_cut, hi_cut, norma ;
    ASLparm *Parm ;
    i = Com->user->i ;
    norma = Com->norma ;
    Parm = Com->Parm ;
    a_is_1 = Parm->a_is_1 ;
    bnd_is_scalar = Parm->bnd_is_scalar ;
    QuadCost = Parm->QuadCost ;
    Subspace = Parm->Subspace ;

    lo_cut = Parm->lo_cut ;
    hi_cut = Parm->hi_cut ;
    if ( bnd_is_scalar )
    {
        Clo = *lo ;
        Chi = *hi ;
    }
    gtd = ASLZERO ;
    err = ASLZERO ;
    nz = 0 ;

    if ( (a == NULL) && !a_is_1 ) /* no linear constraint */
    {
        if ( QuadCost ) /* do not need to evaluate xnew */
        {
            if ( bnd_is_scalar ) 
            {
                for (j = 0; j < n; j++)
                {
                    gj = g [j] ;
                    xj = x [j] ;
                    X = xj - Clo ;
                    Y = Chi - xj ;
                    if ( gj > ASLZERO )
                    {
                        if  ( X > lo_cut )
                        {
                            if ( gj > X ) err = ASLMAX (err, X) ;
                            else          err = ASLMAX (err, gj) ;
                            r = gj/X ;
                            t = -gj/(lambda_k + r) ;
                            if ( Y-t <= hi_cut ) dj = ASLZERO;
                            else                 dj = t ;
                        }
                        else                     dj = ASLZERO ;
                    }
                    else
                    {
                        if ( Y > hi_cut )
                        {
                            r = gj/Y ;
                            t = -gj ;
                            if ( t > Y ) err = ASLMAX (err, Y) ;
                            else         err = ASLMAX (err, t) ;
                            t = t/(lambda_k - r) ;
                            if ( X+t <= lo_cut ) dj = ASLZERO;
                            else                 dj = t ;
                        }
                        else                     dj = ASLZERO ;
                    }
                    /* only store nonzeros in d */
                    if ( dj != ASLZERO )
                    {
                        d [nz] = dj ;
                        i [nz] = j ;
                        gtd += dj*gj ;
                        nz++ ;
                    }
                } 
            }
            else
            {
                for (j = 0; j < n; j++)
                {
                    gj = g [j] ;
                    xj = x [j] ;
                    X = xj - lo [j] ;
                    Y = hi [j] - xj ;
                    if ( gj > ASLZERO )
                    {
                        if  ( X > lo_cut )
                        {
                            if ( gj > X ) err = ASLMAX (err, X) ;
                            else          err = ASLMAX (err, gj) ;
                            r = gj/X ;
                            t = -gj/(lambda_k + r) ;
                            if ( Y-t <= hi_cut ) dj = ASLZERO;
                            else                 dj = t ;
                        }
                        else                     dj = ASLZERO ;
                    }
                    else
                    {
                        if ( Y > hi_cut )
                        {
                            r = gj/Y ;
                            t = -gj ;
                            if ( t > Y ) err = ASLMAX (err, Y) ;
                            else         err = ASLMAX (err, t) ;
                            t = t/(lambda_k - r) ;
                            if ( X+t <= lo_cut ) dj = ASLZERO;
                            else                 dj = t ;
                        }
                        else                     dj = ASLZERO ;
                    }
                    /* only store nonzeros in d */
                    if ( dj != ASLZERO )
                    {
                        d [nz] = dj ;
                        i [nz] = j ;
                        gtd += dj*gj ;
                        nz++ ;
                    }
                }
            }
        }
        else /* not quadratic cost, evaluate xnew */
        {
            if ( bnd_is_scalar ) 
            {
                for (j = 0; j < n; j++)
                {
                    gj = g [j] ;
                    xj = x [j] ;
                    X = xj - Clo ;
                    Y = Chi - xj ;
                    if ( gj > ASLZERO )
                    {
                        if  ( X > lo_cut )
                        {
                            if ( gj > X ) err = ASLMAX (err, X) ;
                            else          err = ASLMAX (err, gj) ;
                            r = gj/X ;
                            t = -gj/(lambda_k + r) ;
                            if ( Y-t <= hi_cut ) dj = ASLZERO;
                            else { dj = t ; xnew [j] = x [j] + t ; }
                        }
                        else dj = ASLZERO ;
                    }
                    else
                    {
                        if ( Y > hi_cut )
                        {
                            r = gj/Y ;
                            t = -gj ;
                            if ( t > Y ) err = ASLMAX (err, Y) ;
                            else         err = ASLMAX (err, t) ;
                            t = t/(lambda_k - r) ;
                            if ( X+t <= lo_cut ) dj = ASLZERO;
                            else { dj = t ; xnew [j] = x [j] + t ; }
                        }
                        else dj = ASLZERO ;
                    }
                    /* only store nonzeros in d */
                    if ( dj != ASLZERO )
                    {
                        d [nz] = dj ;
                        i [nz] = j ;
                        gtd += dj*gj ;
                        nz++ ;
                    }
                } 
            }
            else
            {
                for (j = 0; j < n; j++)
                {
                    gj = g [j] ;
                    xj = x [j] ;
                    X = xj - lo [j] ;
                    Y = hi [j] - xj ;
                    if ( gj > ASLZERO )
                    {
                        if  ( X > lo_cut )
                        {
                            if ( gj > X ) err = ASLMAX (err, X) ;
                            else          err = ASLMAX (err, gj) ;
                            r = gj/X ;
                            t = -gj/(lambda_k + r) ;
                            if ( Y-t <= hi_cut ) dj = ASLZERO;
                            else { dj = t ; xnew [j] = xj + t ; }
                        }
                        else dj = ASLZERO ;
                    }
                    else
                    {
                        if ( Y > hi_cut )
                        {
                            r = gj/Y ;
                            t = -gj ;
                            if ( t > Y ) err = ASLMAX (err, Y) ;
                            else         err = ASLMAX (err, t) ;
                            t = t/(lambda_k - r) ;
                            if ( X+t <= lo_cut ) dj = ASLZERO;
                            else { dj = t ; xnew [j] = xj + t ; }
                        }
                        else dj = ASLZERO ;
                    }
                    /* only store nonzeros in d */
                    if ( dj != ASLZERO )
                    {
                        d [nz] = dj ;
                        i [nz] = j ;
                        gtd += dj*gj ;
                        nz++ ;
                    }
                }
            }
        }
        *Err = err ;
        *Gtd = gtd ;
        Com->user->nz = nz ;
        return (0) ;
    }
    gamma = Parm->gamma ;
    epsmu0 = Parm->epsmu0 ;
    epsmu1 = Parm->epsmu1 ;
    max_mu_its = Parm->max_mu_its ;
    PrintLevel = Parm->PrintLevel ;
    
    /* bracket the root */
    A =  ASLINF ; /* stores lower bound for bracketting interval */
    B = -ASLINF ; /* stores upper bound for bracketting interval */
    normg = ASLZERO ;
    if ( a_is_1 )
    {
        for (j = 0; j < n; j++)
        {
            gj = g [j] ;
            if ( fabs (gj) > normg ) normg = fabs (gj) ;
            if ( gj < A ) A = gj ;
            if ( gj > B ) B = gj ;
        }
    }
    else
    {
        for (j = 0; j < n; j++)
        {
            gj = g [j] ;
            if ( fabs (gj) > normg ) normg = fabs (gj) ;
            aj = a [j] ;
            if ( aj != ASLZERO )
            {
                t = gj/aj ;
                if ( t < A ) A = t ;
                if ( t > B ) B = t ;
            }
        }
    }
    if ( normg == ASLZERO ) /* trivial convergence */
    {
        for (j = 0; j < n; j++) d [j] = ASLZERO ;
        *Err = err ;
        return (0) ;
    }

    C = *mu ;
    /* if the estimate lies outside bracketing interval, use the midpoint */
    if ( (C <= A) || (C >= B) ) C = .5*(A+B) ;

    if ( PrintLevel >= 2 )
    {
        printf ("A: %e C: %e B: %e\n", A, C, B) ;
    }
 
    /* evaluate function and derivative at a, b, and c */
    va = ASLZERO ;
    vb = ASLZERO ;
    vc = ASLZERO ;
    da = ASLZERO ;
    db = ASLZERO ;
    dc = ASLZERO ;

/*printf ("lambda_k: %e\n", lambda_k) ;*/
/*for (j = 0; j < n; j++) printf ("j: %i g: %e\n", j, g [j]) ;*/
    if ( a_is_1 )
    {
        for (j = 0; j < n; j++)
        {
            gj = g [j] ;
            t = gj - A ;
            if ( bnd_is_scalar )
            {
                X = x [j] - Clo ;
                Y = Chi - x [j] ;
            }
            else
            {
                X = x [j] - lo [j] ;
                Y = hi [j] - x [j] ;
            }
            if ( t > ASLZERO )
            {
                if ( X > ASLZERO )
                {
                    r = t/X ;
                    u = lambda_k + r ;
                    va += t/u ;
                    da -= lambda_k/(u*u);
                }
            }
            else
            {
                if ( Y > ASLZERO )
                {
                    r = t/Y ;
                    u = lambda_k - r ;
                    va += t/u ;
                    da -= lambda_k/(u*u) ;
                }
            }
            t = gj - B ;
            if ( t > ASLZERO )
            {
                if ( X > ASLZERO )
                {
                    r = t/X ;
                    u = lambda_k + r ;
                    vb += t/u ;
                    db -= lambda_k/(u*u) ;
                }
            }
            else
            {
                if ( Y > ASLZERO )
                {
                    r = t/Y ;
                    u = lambda_k - r ;
                    vb += t/u ;
                    db -= lambda_k/(u*u) ;
                }
            }
            t = gj - C ;
            if ( t > ASLZERO )
            {
                if ( X > ASLZERO )
                {
                    r = t/X ;
                    u = lambda_k + r ;
                    vc += t/u ;
                    dc -= lambda_k/(u*u) ;
                }
            }
            else
            {
                if ( Y > ASLZERO )
                {
                    r = t/Y ;
                    u = lambda_k - r ;
                    vc += t/u ;
                    dc -= lambda_k/(u*u) ;
                }
            }
        }
    }
    else
    {
        for (j = 0; j < n; j++)
        {
            aj = a [j] ;
            gj = g [j] ;
            t = gj - A*aj ;
            alk = aj*aj*lambda_k ;
            if ( bnd_is_scalar )
            {
                X = x [j] - Clo ;
                Y = Chi - x [j] ;
            }
            else
            {
                X = x [j] - lo [j] ;
                Y = hi [j] - x [j] ;
            }
            if ( t > ASLZERO )
            {
                if ( X > ASLZERO )
                {
                    r = t/X ;
                    u = lambda_k + r ;
                    va += aj*t/u ;
                    da -= alk/(u*u);
                }
            }
            else
            {
                if ( Y > ASLZERO )
                {
                    r = t/Y ;
                    u = lambda_k - r ;
                    va += aj*t/u ;
                    da -= alk/(u*u) ;
                }
            }
            t = gj - B*aj ;
            if ( t > ASLZERO )
            {
                if ( X > ASLZERO )
                {
                    r = t/X ;
                    u = lambda_k + r ;
                    vb += aj*t/u ;
                    db -= alk/(u*u) ;
                }
            }
            else
            {
                if ( Y > ASLZERO )
                {
                    r = t/Y ;
                    u = lambda_k - r ;
                    vb += aj*t/u ;
                    db -= alk/(u*u) ;
                }
            }
            t = gj - C*aj ;
            if ( t > ASLZERO )
            {
                if ( X > ASLZERO )
                {
                    r = t/X ;
                    u = lambda_k + r ;
                    vc += aj*t/u ;
                    dc -= alk/(u*u) ;
                }
            }
            else
            {
                if ( Y > ASLZERO )
                {
                    r = t/Y ;
                    u = lambda_k - r ;
                    vc += aj*t/u ;
                    dc -= alk/(u*u) ;
                }
            }
        }
    }
    tol0 = epsmu0*normg/norma ;

    if ( (va < ASLZERO) || (vb > ASLZERO) )
    {
        printf ("computation of muk: either va = %e < 0 or vb = %e > 0\n",
                 va, vb) ;
        return (5) ;
    }

    if ( vc > ASLZERO ) /* if vc > 0 move right */
    {
        A = C ;
        va = vc ;
        da = dc ;
    }
    else             /* move left */
    {
        B = C ;
        vb = vc ;
        db = dc ;
    }

    method = 0 ;                             /* start with Newton's method */
    width = ASLINF ;
    iu = 0 ;
    if ( va <= ASLZERO )
    {
        C = A ;
        goto Exit ;
    }
    else if ( vb >= ASLZERO )
    {
        C = B ;
        goto Exit ;
    }
    prior_side = 0 ;
    ok = ASLTRUE ;
    for (iu = 0; iu < max_mu_its; iu++)
    {
        /* test for convergence */
        tol1 = epsmu1*ASLMAX (fabs (A), fabs (B)) ;
        t = B - A ;
/*printf ("t: %e tol0: %e tol1: %e method: %i\n", t, tol0, tol1, method) ;*/
        if ( (((t <= tol0) || (t <= tol1)) && ok)
             || (va == ASLZERO) || (vb == ASLZERO) )
        {
/*printf ("terminate at top\n") ;*/
            if ( va > -vb )
            {
                C = B ;
                vc = vb ;
            }
            else
            {
                C = A ;
                vc = va ;
            }
            goto Exit ;
        }

        ok = ASLTRUE ;
        /* if the previous method was secant and b-a > width, use bisection */
        if ( (method == 0) && (B-A > width) )
        {
            C = .5*(A+B) ;
            width = ASLINF ;                  /* ensures Newton is used next */
            prior_side = 0 ;
        }
        else if ( va > -vb )
        {
/*printf ("use right side\n") ;*/
            if ( method == 0 )               /* Newton */
            {
                width = gamma*(B-A) ;        /* Secant must give b-a <= width */
                C = B - vb/db ;              /* Newton from b */
/*printf ("newton from B\n") ;*/
                /* bisect if Newton outside [A,B]*/
                if ( (C >= B) || (C <= A) ) C = .5*(A+B) ;
                method = 1 ;                 /* switch to Secant */
            }
            else                             /* Secant */
            {
                if ( iu == 1 )
                {
                    if ( va/vb <= -1.e3 )
                    {
                        C = B - 2*vb/db ;
                        if ( C <= A ) C = B - (A-B)*(vb/(va-vb)) ;
                    }
                    else C = B - (A-B)*(vb/(va-vb)) ; /*Secant from b*/
                }
                else
                {
                    C = B - (A-B)*(vb/(va-vb)) ; /* Secant from b */
                }
/*printf ("secant from B\n") ; */
                method = 0 ;                 /* switch to Newton */
            }
        }
        else
        {
/*printf ("use left side\n") ; */
            if ( method == 0 )               /* Newton */
            {
                width = gamma*(B-A) ;        /* Secant must give b-a <= width */
                C = A - va/da ;              /* Newton from a */
/*printf ("newton from A\n") ; */
                /* bisect if Newton outside [A,B]*/
                if ( (C >= B) || (C <= A) ) C = .5*(A+B) ;
                method = 1 ;                 /* switch to Secant */
            }
            else                             /* Secant */
            {
                if ( iu == 1 )
                {
                    if ( vb/va <= -1.e3 )
                    {
                        C = A - 2*va/da ;
                        if ( C >= B ) C = A - (A-B)*(va/(va-vb)) ;
                    }
                    else C = A - (A-B)*(va/(va-vb)) ; /*Secant from a*/
                }
                else
                {
                    C = A - (A-B)*(va/(va-vb)) ; /* Secant from a */
                }
/*printf ("secant from A\n") ;*/
                method = 0 ;                 /* switch to Newton */
            }
        }

        if ( PrintLevel == 2 )
        {
            if ( width == ASLINF )
            {
                printf ("Bisect A: %12.5e va: %12.5e C: %12.5e B: "
                        "%12.5e vb: %12.5e\n", A, va, C, B, vb) ;
            }
            else if ( method == 1 )
            {
                printf ("Newton A: %12.5e va: %12.5e C: %12.5e B: "
                        "%12.5e vb: %12.5e\n", A, va, C, B, vb) ;
            }
            else
            {
                printf ("Secant A: %12.5e va: %12.5e C: %12.5e B: "
                        "%12.5e vb: %12.5e\n", A, va, C, B, vb) ;
            }
        }
        /* evaluate function and derivative */
        ASLval (&vc, &dc, C, lambda_k, x, g, a, lo, hi, n, Com) ;
        if ( vc > ASLZERO ) /* move right */
        {
/*printf ("A is updated, prior_side: %i\n", prior_side) ;  */
            if ( C <= A ) goto Exit ;
            if ( prior_side == -1 ) /* left side updated twice */
            {
/*printf ("prior side updated twice\n") ; */
                D = ASLMIN (A + tol0, .5*(A+B)) ;
/*printf ("D: %e\n", D) ; */
                if ( D > C )
                {
/*printf ("creep to %e\n", t) ; */
                    ASLval (&vd, &dd, D, lambda_k, x, g, a, lo, hi, n, Com);
                    if ( vd >= ASLZERO ) /* update left side */
                    {
                        A = D ;
                        va = vd ;
                        da = dd ;
                    }
                    else /* update right side and flag for Newton step */
                    {
                        prior_side = 1 ;
                        B = D ;
                        vb = vd ;
                        db = dd ;
                        A = C ;
                        va = vc ;
                        da = dc ;
                        width = ASLINF ;
                        method = 0 ;
                        ok = ASLFALSE ;
/* printf ("set method to newton\n") ;*/
                    }
                }
                else
                {
                    A = C ;
                    va = vc ;
                    da = dc ;
                }
            }
            else
            {
                prior_side = -1 ;
                A = C ;
                va = vc ;
                da = dc ;
            }
        }
        else             /* move left */
        {
/*printf ("B is updated, prior side: %i\n", prior_side) ;*/
            if ( C >= B ) goto Exit ;
            if ( prior_side == 1 ) /* right side updated twice */
            {
                D = ASLMAX (B - tol0, .5*(A+B)) ;
                if ( D < C )
                {
/* printf ("creep to %e\n", t) ;*/
                    ASLval (&vd, &dd, D, lambda_k, x, g, a, lo, hi, n, Com);
                    if ( vd <= ASLZERO ) /* update right side */
                    {
                        B = D ;
                        vb = vd ;
                        db = dd ;
                    }
                    else /* update left side and flag for Newton step */
                    {
/* printf ("set method to newton\n") ;*/
                        prior_side = -1 ;
                        A = D ;
                        va = vd ;
                        da = dd ;
                        B = C ;
                        vb = vc ;
                        db = dc ;
                        width = ASLINF ;
                        method = 0 ;
                        ok = ASLFALSE ;
                    }
                }
                else
                {
                    B = C ;
                    vb = vc ;
                    db = dc ;
                }
            }
            else
            {
                prior_side = 1 ;
                B = C ;
                vb = vc ;
                db = dc ;
            }
        }
        if ( va <= ASLZERO)
        {
            C = A ;
            goto Exit ;
        }
        else if ( vb >= ASLZERO )
        {
            C = B ;
            goto Exit ;
        }
    }
    return (7) ;
    Exit:
    if ( PrintLevel >= 2 )
    {
        printf ("Mu iterations: %6i A: %12.5e C: %12.5e B: %12.5e vc: %12.5e\n",
                 iu, A, C, B, vc) ;
    }
    Com->nmu += iu ;
    if ( QuadCost ) /* do not need to evaluate xnew */
    {
        if ( bnd_is_scalar ) 
        {
            for (j = 0; j < n; j++)
            {
                gj = g [j] ;
                if ( a_is_1 ) t = gj - C ;
                else          t = gj - C*a [j] ;
                xj = x [j] ;
                X = xj - Clo ;
                Y = Chi - xj ;

                if ( t > ASLZERO )
                {
                    if  ( X > lo_cut )
                    {
                        if ( t > X ) err = ASLMAX (err, X) ;
                        else         err = ASLMAX (err, t) ;
                        r = t/X ;
                        t = -t/(lambda_k + r) ;
                        /* if t is small, xj could be upper bound active */
                        if ( Y-t <= hi_cut ) dj = ASLZERO ;
                        else                 dj = t ;
                    }
                    else                     dj = ASLZERO ;
                }
                else
                {
                    if ( Y > hi_cut )
                    {
                        r = t/Y ;
                        t = -t ;
                        if ( t > Y ) err = ASLMAX (err, Y) ;
                        else         err = ASLMAX (err, t) ;
                        t = t/(lambda_k - r) ;
                        if ( X+t <= lo_cut ) dj = ASLZERO ;
                        else                 dj = t ;
                    }
                    else                     dj = ASLZERO ;
                }
                /* only store nonzeros in d */
                if ( dj != ASLZERO )
                {
                    d [nz] = dj ;
                    if ( Subspace ) i [nz] = i [j] ;
                    else            i [nz] = j ;
                    gtd += dj*gj ;
                    nz++ ;
                }
            } 
        }
        else
        {
            for (j = 0; j < n; j++)
            {
                gj = g [j] ;
                if ( a_is_1 ) t = gj - C ;
                else          t = gj - C*a [j] ;
                xj = x [j] ;
                X = xj - lo [j] ;
                Y = hi [j] - xj ;
                if ( t > ASLZERO )
                {
                    if  ( X > lo_cut )
                    {
                        if ( t > X ) err = ASLMAX (err, X) ;
                        else         err = ASLMAX (err, t) ;
                        r = t/X ;
                        t = -t/(lambda_k + r) ;
                        if ( Y-t <= hi_cut ) dj = ASLZERO ;
                        else                 dj = t ;
                    }
                    else                     dj = ASLZERO ;
                }
                else
                {
                    if ( Y > hi_cut )
                    {
                        r = t/Y ;
                        t = -t ;
                        if ( t > Y ) err = ASLMAX (err, Y) ;
                        else         err = ASLMAX (err, t) ;
                        t = t/(lambda_k - r) ;
                        if ( X+t <= lo_cut ) dj = ASLZERO ;
                        else                 dj = t ;
                    }
                    else                     dj = ASLZERO ;
                }
                /* only store nonzeros in d */
                if ( dj != ASLZERO )
                {
                    d [nz] = dj ;
                    if ( Subspace ) i [nz] = i [j] ;
                    else            i [nz] = j ;
                    gtd += dj*gj ;
                    nz++ ;
                }
            }
        }
    }
    else /* cost function not quadratic, evaluate xnew */
    {
        if ( bnd_is_scalar ) 
        {
            for (j = 0; j < n; j++)
            {
                gj = g [j] ;
                if ( a_is_1 ) t = gj - C ;
                else          t = gj - C*a [j] ;
                xj = x [j] ;
                X = xj - Clo ;
                Y = Chi - xj ;
                if ( t > ASLZERO )
                {
                    if  ( X > lo_cut )
                    {
                        if ( t > X ) err = ASLMAX (err, X) ;
                        else         err = ASLMAX (err, t) ;
                        r = t/X ;
                        t = -t/(lambda_k + r) ;
                        if ( Y-t <= hi_cut ) dj = ASLZERO ;
                        else { dj = t ; xnew [j] = xj + t ; }
                    }
                    else dj = ASLZERO ;
                }
                else
                {
                    if ( Y > hi_cut )
                    {
                        r = t/Y ;
                        t = -t ;
                        if ( t > Y ) err = ASLMAX (err, Y) ;
                        else         err = ASLMAX (err, t) ;
                        t = t/(lambda_k - r) ;
                        if ( X+t <= lo_cut ) dj = ASLZERO ;
                        else { dj = t ; xnew [j] = xj + t ; }
                    }
                    else dj = ASLZERO ;
                }
                /* only store nonzeros in d */
                if ( dj != ASLZERO )
                {
                    d [nz] = dj ;
                    if ( Subspace ) i [nz] = i [j] ;
                    else            i [nz] = j ;
                    gtd += dj*gj ;
                    nz++ ;
                }
            } 
        }
        else
        {
            for (j = 0; j < n; j++)
            {
                gj = g [j] ;
                if ( a_is_1 ) t = gj - C ;
                else          t = gj - C*a [j] ;
                xj = x [j] ;
                X = xj - lo [j] ;
                Y = hi [j] - xj ;
                if ( t > ASLZERO )
                {
                    if  ( X > lo_cut )
                    {
                        if ( t > X ) err = ASLMAX (err, X) ;
                        else         err = ASLMAX (err, t) ;
                        r = t/X ;
                        t = -t/(lambda_k + r) ;
                        if ( Y-t <= hi_cut ) dj = ASLZERO ;
                        else { dj = t ; xnew [j] = xj + t ; }
                    }
                    else dj = ASLZERO ;
                }
                else
                {
                    if ( Y > hi_cut )
                    {
                        r = t/Y ;
                        t = -t ;
                        if ( t > Y ) err = ASLMAX (err, Y) ;
                        else         err = ASLMAX (err, t) ;
                        t = t/(lambda_k - r) ;
                        if ( X+t <= lo_cut ) dj = ASLZERO ;
                        else { dj = t ; xnew [j] = xj + t ; }
                    }
                    else dj = ASLZERO ;
                }
                /* only store nonzeros in d */
                if ( dj != ASLZERO )
                {
                    d [nz] = dj ;
                    if ( Subspace ) i [nz] = i [j] ;
                    else            i [nz] = j ;
                    gtd += dj*gj ;
                    nz++ ;
                }
            }
        }
    }
    *Err = err ;
    *Gtd = gtd ;
    *mu = C ;
    Com->user->nz = nz ;
    return (0) ;
}
/* ==========================================================================
   === ASLval ===============================================================
   ==========================================================================
       Evaluate r(C)  and r'(C) where r (C) = a'd (C)
   ========================================================================== */
PRIVATE void ASLval
(
    ASLFLOAT      *vC,
    ASLFLOAT      *dC,
    ASLFLOAT        C,
    ASLFLOAT lambda_k,
    ASLFLOAT       *x,
    ASLFLOAT       *g,
    ASLFLOAT       *a,
    ASLFLOAT      *lo,
    ASLFLOAT      *hi,
    int             n,
    ASLcom       *Com  /* common variables */
)
{
    int j, bnd_is_scalar, a_is_1 ;
    ASLFLOAT aj, alk, r, t, u, vc, dc, X, Clo, Chi, lo_cut, hi_cut ;
    ASLparm *Parm ;
    Parm = Com->Parm ;
    bnd_is_scalar = Parm->bnd_is_scalar ;
    a_is_1 = Parm->a_is_1 ;
    vc = ASLZERO ;
    dc = ASLZERO ;
    if ( bnd_is_scalar )
    {
        Clo = *lo ;
        Chi = *hi ;
    }
    lo_cut = Parm->lo_cut ;
    hi_cut = Parm->hi_cut ;
    if ( a_is_1 )
    {
        for (j = 0; j < n; j++)
        {
            t = g [j] - C ;
            if ( t > ASLZERO )
            {
                if ( bnd_is_scalar ) X = x [j] - Clo ;
                else                 X = x [j] - lo [j] ;
                if ( X > lo_cut )
                {
                    r = t/X ;
                    u = lambda_k + r ;
                    vc += t/u ;
                    dc -= lambda_k/(u*u) ;
                }
            }
            else
            {
                if ( bnd_is_scalar ) X = Chi - x [j] ;
                else                 X = hi [j] - x [j] ;
                if ( X > hi_cut )
                {
                    r = t/X ;
                    u = lambda_k - r ;
                    vc += t/u ;
                    dc -= lambda_k/(u*u) ;
                }
            }
        }
    }
    else
    {
        for (j = 0; j < n; j++)
        {
            aj = a [j] ;
            t = g [j] - C*aj ;
            alk = aj*aj*lambda_k ;
            if ( t > ASLZERO )
            {
                if ( bnd_is_scalar ) X = x [j] - Clo ;
                else                 X = x [j] - lo [j] ;
                if ( X > lo_cut )
                {
                    r = t/X ;
                    u = lambda_k + r ;
                    vc += aj*t/u ;
                    dc -= alk/(u*u) ;
                }
            }
            else
            {
                if ( bnd_is_scalar ) X = Chi - x [j] ;
                else                 X = hi [j] - x [j] ;
                if ( X > hi_cut )
                {
                    r = t/X ;
                    u = lambda_k - r ;
                    vc += aj*t/u ;
                    dc -= alk/(u*u) ;
                }
            }
        }
    }
    *vC = vc ;
    *dC = dc ;
}

/* ==========================================================================
   === ASLbb ================================================================
   ==========================================================================
       Compute lambda_k using BB formula if t > 0,
       If t > 0, then BB scalar is updated, otherwise t = - ASLONE
   ========================================================================== */
PRIVATE ASLFLOAT ASLbb
(
    ASLFLOAT alpha,
    int         mm, /* stop current cycle when Parm->MaxCycle - mm <= 0 */
    ASLcom    *Com  /* common variables */
)
{
    int j, k, n, nz, bnd_is_scalar, Subspace, *i ;
    ASLFLOAT lambda0, dtd, dk, sy, ss, normg, normx, Clo, Chi,
            lo_cut, hi_cut, t, *x, *lo, *hi, *d, *g, *xold, *gold ;
    ASLobjective *Objective ;
    ASLparm *Parm ;
    Parm = Com->Parm ;
    n = Com->n ;
    lambda0 = Parm->lambda0 ;
    Subspace = Parm->Subspace ;
    if ( Subspace )
    {
        bnd_is_scalar = Parm->bnd_is_scalar ;
        lo = Com->lo ;
        hi = Com->hi ;
        if ( bnd_is_scalar )
        {
            Clo = *lo ;
            Chi = *hi ;
        }
        lo_cut = Parm->lo_cut ;
        hi_cut = Parm->hi_cut ;
    }
    Objective = Com->user ;
    d = Objective->d ;
    nz= Objective->nz ;
    i = Objective->i ;
    x = Objective->x ;
    g = Objective->g ;
    xold = Objective->xold ;
    gold = Objective->gold ;
    sy = ASLZERO ;
    ss = ASLZERO ;
    dtd= ASLZERO ;
    if ( Parm->QuadCost )
    {
        sy = Com->deriv2 ; /* d'Ad (the alpha's cancel) */
        for (k = 0; k < nz; k++)
        {
            dk = d [k] ;
            ss += dk*dk ;
            j = i [k] ;
            xold [j] += alpha*dk ;
        }
        /* update g */
        ASLstep (gold, gold, Objective->p, alpha, n) ;
    }
    else
    {
        for (k = 0; k < nz; k++)
        {
            dk = d [k] ;
            ss += dk*dk ;
            j = i [k] ;
            sy += dk*(g [j] - gold [j]) ;
        }
        ss *= alpha ;
    }
    t = -ASLONE ;
    if ( sy <= ASLZERO || ss == ASLZERO )
    {
       if ( Parm->MaxCycle - mm <= 0 ) 
       {
          if ( Parm->QuadCost )
          {
              normg = ASLmax (gold, n) ;
              normx = ASLmax (xold, n) ;
          }
          else
          {
              normg = ASLmax (g, n) ;
              normx = ASLmax (x, n) ;
          }
          if ( normx > ASLZERO ) t = ASLMAX (normg/normx, lambda0) ;
          else                   t = lambda0 ;
          t = ASLMAX (t, Parm->lambda0Factor*normg) ;
       }
    }
    else
    {
       t = ASLMAX (sy/ss, lambda0) ;
       if ( Parm->QuadCost ) normg = ASLmax (gold, n) ;
       else                  normg = ASLmax (g, n) ;
       t = ASLMAX (t, Parm->lambda0Factor*normg) ;
    }
    return (t) ;
}

/* =========================================================================
   === ASLgather ===========================================================
   =========================================================================
   compress x by extracting the subspace elements and copying them to
   the compressed array y
   ========================================================================= */
PRIVATE void ASLgather
(
    ASLFLOAT *y, /* compressed vector */
    ASLFLOAT *x, /* current x */
    int      *i, /* indices associated with subspace */
    int       n  /* number of elements to extract from x */
)
{
    int k, n5, *ik ;
    ASLFLOAT *yk ;
    n5 = n % 5 ;
    for (k = 0; k < n5; k++) y [k] = x [i [k]] ;
    yk = y+k ;
    ik = i+k ;
    for (; k < n; k += 5)
    {
        *(yk++) = x [*(ik++)] ;
        *(yk++) = x [*(ik++)] ;
        *(yk++) = x [*(ik++)] ;
        *(yk++) = x [*(ik++)] ;
        *(yk++) = x [*(ik++)] ;
    }
}

/* =========================================================================
   === ASLscatter ==========================================================
   =========================================================================
   scatter elements from the compressed array y into x
   ========================================================================= */
PRIVATE void ASLscatter
(
    ASLFLOAT *x, /* scatter y into x */
    ASLFLOAT *y, /* compressed vector */
    int      *i, /* indices associated with subspace */
    int       n  /* number of elements to extract from x */
)
{
    int k, n5, *ik ;
    ASLFLOAT *yk ;
    n5 = n % 5 ;
    for (k = 0; k < n5; k++) x [i [k]] = y [k] ;
    yk = y+k ;
    ik = i+k ;
    for (; k < n; k += 5)
    {
        x [*(ik++)] = *(yk++) ;
        x [*(ik++)] = *(yk++) ;
        x [*(ik++)] = *(yk++) ;
        x [*(ik++)] = *(yk++) ;
        x [*(ik++)] = *(yk++) ;
    }
}

/* =========================================================================
   === ASLstep =============================================================
   =========================================================================
   set xnew = x + st*d
   ========================================================================= */
PRIVATE void ASLstep
(
    ASLFLOAT *xnew, /* updated x vector */
    ASLFLOAT    *x, /* current x */
    ASLFLOAT    *d, /* search direction */
    ASLFLOAT    st, /* stepsize */
    int          n  /* dimension */
)
{
    int j, n5 ;
    n5 = n % 5 ;
    if ( st == ASLONE )
    {
        for (j = 0; j < n5; j++) xnew [j] = x [j] + d [j] ;
        for (; j < n; j += 5)
        {
            xnew [j  ] = x [j  ] + d [j  ] ;
            xnew [j+1] = x [j+1] + d [j+1] ;
            xnew [j+2] = x [j+2] + d [j+2] ;
            xnew [j+3] = x [j+3] + d [j+3] ;
            xnew [j+4] = x [j+4] + d [j+4] ;
        }
    }
    else
    {
        for (j = 0; j < n5; j++)
        {
            xnew [j] = x [j] + st*d [j] ;
        }
        for (; j < n; j += 5)
        {
            xnew [j  ] = x [j  ] + st*d [j  ] ;
            xnew [j+1] = x [j+1] + st*d [j+1] ;
            xnew [j+2] = x [j+2] + st*d [j+2] ;
            xnew [j+3] = x [j+3] + st*d [j+3] ;
            xnew [j+4] = x [j+4] + st*d [j+4] ;
        }
    }
}

/* =========================================================================
   === ASLstepi ============================================================
   =========================================================================
   set xnew_j = x_j + st*d_j, j = i [k], 0 <= k < nz
   ========================================================================= */
PRIVATE void ASLstepi
(
    ASLFLOAT *xnew, /* updated x vector */
    int         *i, /* indices of xnew that change */
    ASLFLOAT    *x, /* current x */
    ASLFLOAT    *d, /* search direction */
    ASLFLOAT    st, /* stepsize */
    int         nz  /* number of components to be updated */
)
{
    int j, k, n5, *jk ;
    n5 = nz % 5 ;
    jk = i ;
    if ( st == ASLONE )
    {
        for (k = 0; k < n5; k++)
        {
            j = *(jk++) ;
            xnew [j] = x [j] + d [k] ;
        }
        for (; k < nz; )
        {
            j = *(jk++) ;
            xnew [j] = x [j] + d [k++] ;
            j = *(jk++) ;
            xnew [j] = x [j] + d [k++] ;
            j = *(jk++) ;
            xnew [j] = x [j] + d [k++] ;
            j = *(jk++) ;
            xnew [j] = x [j] + d [k++] ;
            j = *(jk++) ;
            xnew [j] = x [j] + d [k++] ;
        }
    }
    else
    {
        for (k = 0; k < n5; )
        {
            j = *(jk++) ;
            xnew [j] = x [j] + st*d [k++] ;
        }
        for (; k < nz; )
        {
            j = *(jk++) ;
            xnew [j] = x [j] + st*d [k++] ;
            j = *(jk++) ;
            xnew [j] = x [j] + st*d [k++] ;
            j = *(jk++) ;
            xnew [j] = x [j] + st*d [k++] ;
            j = *(jk++) ;
            xnew [j] = x [j] + st*d [k++] ;
            j = *(jk++) ;
            xnew [j] = x [j] + st*d [k++] ;
        }
    }
}

/* =========================================================================
   === ASLdot ==============================================================
   =========================================================================
   Compute dot product of x and y
   ========================================================================= */
PRIVATE ASLFLOAT ASLdot
(
    ASLFLOAT *x, /* first vector */
    ASLFLOAT *y, /* second vector */
    int       n  /* length of vectors */
)
{
    int j, n5 ;
    ASLFLOAT t ;
    t = ASLZERO ;
    n5 = n % 5 ;
    for (j = 0; j < n5; j++) t += x [j]*y [j] ;
    for (; j < n; j += 5)
    {
        t += x [j]*y[j] + x [j+1]*y [j+1] + x [j+2]*y [j+2]
                        + x [j+3]*y [j+3] + x [j+4]*y [j+4] ;
    }
    return (t) ;
}

/* =========================================================================
   === ASLdoti =============================================================
   =========================================================================
   Compute dot product of x and y, x is sparse and y is dense
   ========================================================================= */
PRIVATE ASLFLOAT ASLdoti
(
    ASLFLOAT *x, /* first vector */
    int      *i, /* indices associated with elements of x */
    ASLFLOAT *y, /* second vector */
    int       n  /* length of x */
)
{
    int j, n5, *ij ;
    ASLFLOAT t, *xj ;
    t = ASLZERO ;
    n5 = n % 5 ;
    for (j = 0; j < n5; j++) t += x [j]*y [i [j]] ;
    for (; j < n; j += 5)
    {
        ij = i+j ;
        xj = x+j ;
        t += xj[0]*y[ij[0]] + xj[1]*y[ij[1]] + xj[2]*y[ij[2]]
                            + xj[3]*y[ij[3]] + xj[4]*y[ij[4]] ;
    }
    return (t) ;
}

/* =========================================================================
   === ASLadd ==============================================================
   =========================================================================
   add the components of a vector
   ========================================================================= */
double ASLadd
(
    ASLFLOAT *x, /* vector */
    int       n  /* length of vector */
)
{
    int j, n5 ;
    ASLFLOAT t ;

    t = 0. ;
    n5 = n % 5 ;
    for (j = 0; j < n5; j++) t += x [j] ;
    for (; j < n; j += 5)
    {
        t += x [j] + x [j+1] + x [j+2] + x [j+3] + x [j+4] ;
    }
    return (t) ;
}

/* =========================================================================
   === ASLcopy ==============================================================
   =========================================================================
   Copy the second vector to the first x = y
   ========================================================================= */
PRIVATE void ASLcopy
(
    ASLFLOAT *x, /* copy of y */
    ASLFLOAT *y, /* given vector */
    int       n  /* length of vectors */
)
{
    int j, n5 ;
    n5 = n % 5 ;
    for (j = 0; j < n5; j++) x [j] = y [j] ;
    for (; j < n; j += 5)
    {
        x [j]   =  y[j] ;
        x [j+1] =  y[j+1] ;
        x [j+2] =  y[j+2] ;
        x [j+3] =  y[j+3] ;
        x [j+4] =  y[j+4] ;
    }
}

/* =========================================================================
   === ASLmax ==============================================================
   =========================================================================
   Return max {fabs (x [j]) : 1 <= j < n}
   ========================================================================= */
PRIVATE ASLFLOAT ASLmax
(
    ASLFLOAT *x,
    int       n
)
{
    ASLFLOAT xnorm ;
    int j, n5 ;
    n5 = n % 5 ;
    xnorm = ASLZERO ;
    for (j = 0; j < n5; j++) if ( xnorm < fabs (x [j]) ) xnorm = fabs (x [j]) ;
    for (; j < n; j += 5)
    {
        if ( xnorm < fabs (x [j]  ) ) xnorm = fabs (x [j]) ;
        if ( xnorm < fabs (x [j+1]) ) xnorm = fabs (x [j+1]) ;
        if ( xnorm < fabs (x [j+2]) ) xnorm = fabs (x [j+2]) ;
        if ( xnorm < fabs (x [j+3]) ) xnorm = fabs (x [j+3]) ;
        if ( xnorm < fabs (x [j+4]) ) xnorm = fabs (x [j+4]) ;
    }
    return (xnorm) ;
}

/* =========================================================================
   === ASLint_init =========================================================
   =========================================================================
   Initialize an array
   ========================================================================= */
void ASLint_init
(
    int *x,  /* array to be initialized */
    int  s,  /* scalar */
    int  n   /* length of x */
)
{
    int j, n5 ;
    int *xj ;
    n5 = n % 5 ;
    for (j = 0; j < n5; j++) x [j] = s ;
    xj = x+j ;
    for (; j < n; j += 5)
    {
        *(xj++) = s ;
        *(xj++) = s ;
        *(xj++) = s ;
        *(xj++) = s ;
        *(xj++) = s ;
    }
}

/* ==========================================================================
   === ASLpartialMinSort ====================================================
   ==========================================================================
   find indices of m smallest numbers in a list, sort in increasing order
   ========================================================================== */
PRIVATE void ASLpartialMinSort
(
    int      *i, /* indices of m smallest numbers */
    ASLFLOAT *x, /* numbers to sort */
    int       m, /* length of i */
    int       n  /* length of x */
)
{
    int j ;
    ASLFLOAT top ;
    if ( m >= n )
    {
        for (j = 0; j < n; j++) i [j] = j ;
        return ;
    }
    for (j = 0; j < m; j++) i [j] = j ;
    ASLmaxheap_build (i-1, x, m) ;
    top = x [i [0]] ;
    for (; j < n; j++)
    {
        if ( top > x [j] )
        {
            i [0] = j ;
            ASLmaxheapify (i-1, x, 1, m) ;
            top = x [i [0]] ;
        }
    }
}

/* ==========================================================================
   === ASLpartialMaxSort ====================================================
   ==========================================================================
   find indices of m largest numbers in a list
   ========================================================================== */
PRIVATE void ASLpartialMaxSort
(
    int      *i, /* indices of m largest numbers */
    ASLFLOAT *x, /* numbers to sort */
    int       m, /* length of i */
    int       n  /* length of x */
)
{
    int j ;
    ASLFLOAT bot ;
    if ( m >= n )
    {
        for (j = 0; j < n; j++) i [j] = j ;
        return ;
    }
    for (j = 0; j < m; j++) i [j] = j ;
    ASLminheap_build (i-1, x, m) ;
    bot = x [i [0]] ;
    for (; j < n; j++)
    {
        if ( bot < x [j] )
        {
            i [0] = j ;
            ASLminheapify (i-1, x, 1, m) ;
            bot = x [i [0]] ;
        }
    }
}

/* ==========================================================================
   === ASLminHeapSort =======================================================
   ==========================================================================
   given a min heap, generate the list of indices corresponding to decreasing
   order
   ========================================================================== */
PRIVATE void ASLminHeapSort
(
    int      *i, /* indices with decreasing order based on x */
    int   *heap, /* min heap */
    ASLFLOAT *x, /* numbers to sort */
    int       n  /* size of heap */
)
{
    int j, k, l ;
    l = n ;
    for (j = 0; j < l; j++)
    {
        k = heap [0] ;
        n = ASLminheap_delete (heap-1, x, n) ;
        i [n] = k ;        /* smallest goes to end of i */
    }
}

/* ==========================================================================
   === ASLmaxHeapSort =======================================================
   ==========================================================================
   given a max heap, generate the list of indices corresponding to increasing
   order
   ========================================================================== */
PRIVATE void ASLmaxHeapSort
(
    int      *i, /* indices with increasing order based on x */
    int   *heap, /* max heap */
    ASLFLOAT *x, /* numbers to sort */
    int       n  /* size of heap */
)
{
    int j, k, l ;
    l = n ;
    for (j = 0; j < l; j++)
    {
        k = heap [0] ;
        n = ASLmaxheap_delete (heap-1, x, n) ;
        i [n] = k ;        /* biggest goes to end of i */
    }
}

/* ==========================================================================
   === minheap_build ========================================================
   ==========================================================================
   build a min heap in heap [1..nheap]
   heap [i] is an index of an element of array x
   ========================================================================== */

PRIVATE void ASLminheap_build
(
    int   *heap, /* on input, an unsorted set of indices */
    ASLFLOAT *x, /* numbers to sort */
    int       n  /* number of elements to build into the heap */
)
{
    int p ;

    for (p = n/2 ; p >= 1 ; p--)
    {
        ASLminheapify (heap, x, p, n) ;
    }
}

/* ==========================================================================
   === minheap_delete =======================================================
   ==========================================================================
   delete the top element in a min heap
   ========================================================================== */
PRIVATE int ASLminheap_delete  /* return new size of heap */
(
    int   *heap, /* containing indices into x, 1..n on input */
    ASLFLOAT *x, /* not modified */
    int       n  /* number of items in heap */
)
{
    if (n <= 1)
    {
        return (0) ;
    }

    /* move element from the end of the heap to the top */
    heap [1] = heap [n] ;
    n-- ;
    ASLminheapify (heap, x, 1, n) ;
    return (n) ;
}

/* ========================================================================== */
/* === minheap_add ========================================================== */
/* ========================================================================== */
/* ========================================================================== */

/* add a new leaf to a min heap */

PRIVATE int ASLminheap_add
(
    int    leaf, /* the new leaf */
    int   *heap, /* size n, containing indices into x */
    ASLFLOAT *x, /* not modified */
    int       n  /* number of elements in heap not counting new one */
)
{
    int l, inew, iold ;
    ASLFLOAT xold, xnew ;

    n++ ;
    iold = n ;
    heap [iold] = leaf ;
    xold = x [leaf] ;
    while ( iold > 1 )
    {
        inew = iold/2 ;
        l = heap [inew] ;
        xnew = x [l] ;
        if ( xnew > xold )
        { /* swap new and old */
            heap [inew] = leaf ;
            heap [iold] = l ;
        }
        else return (n) ;
        iold = inew ;
    }
    return (n) ;
}

/* ========================================================================== */
/* === minheapify =========================================================== */
/* ========================================================================== */

/* heapify starting at node p.  On input, the heap at node p satisfies the */
/* heap property, except for heap [p] itself.  On output, the whole heap */
/* satisfies the heap property. */

PRIVATE void ASLminheapify
(
    int   *heap, /* size n, containing indices into x */
    ASLFLOAT *x, /* not modified */
    int       p, /* start at node p in the heap */
    int       n  /* heap [1 ... n] is in use */
)
{
    int left, right, e, hleft, hright ;
    ASLFLOAT xe, xleft, xright ;

    e = heap [p] ;
    xe = x [e] ;

    while ( ASLTRUE )
    {
        left = p * 2 ;
        right = left + 1 ;

        if ( right <= n )
        {
            hleft  = heap [left] ;
            hright = heap [right] ;
            xleft  = x [hleft] ;
            xright = x [hright] ;
            if (xleft < xright)
            {
                if (xe > xleft)
                {
                    heap [p] = hleft ;
                    p = left ;
                }
                else
                {
                    heap [p] = e ;
                    return ;
                }
            }
            else
            {
                if (xe > xright)
                {
                    heap [p] = hright ;
                    p = right ;
                }
                else
                {
                    heap [p] = e ;
                    return ;
                }
            }
        }
        else
        {
            if ( left <= n )
            {
                hleft = heap [left] ;
                xleft = x [hleft] ;
                if (xe > xleft)
                {
                    heap [p] = hleft ;
                    p = left ;
                }
            }
            heap [p] = e ;
            return ;
        }
    }
}

/* ========================================================================== */
/* === maxheap_build ======================================================== */
/* ========================================================================== */

/* build a max heap in heap [1..n] */

PRIVATE void ASLmaxheap_build
(
    int   *heap, /* on input, an unsorted set of elements */
    ASLFLOAT *x,
    int       n  /* number of elements to build into the heap */
)
{
    int p ;

    for (p = n/2 ; p >= 1 ; p--)
    {
        ASLmaxheapify (heap, x, p, n) ;
    }
}

/* ========================================================================== */
/* === maxheap_delete ======================================================= */
/* ========================================================================== */

/* delete the top element in a max heap */

PRIVATE int ASLmaxheap_delete  /* return new size of heap */
(
    int   *heap, /* containing indices into x, 1..n on input */
    ASLFLOAT *x, /* not modified */
    int       n  /* number of items in heap */
)
{
    if (n <= 1)
    {
        return (0) ;
    }

    /* move element from the end of the heap to the top */
    heap [1] = heap [n] ;
    n-- ;
    ASLmaxheapify (heap, x, 1, n) ;
    return (n) ;
}

/* ========================================================================== */
/* === maxheap_add ========================================================== */
/* ========================================================================== */

/* add a new leaf to a max heap */

PRIVATE int ASLmaxheap_add
(
    int   *heap, /* size n, containing indices into x */
    ASLFLOAT *x, /* not modified */
    int    leaf, /* the new leaf */
    int       n  /* number of elements in heap not counting new one */
)
{
    int l, inew, iold ;
    ASLFLOAT xold, xnew ;

    n++ ;
    iold = n ;
    heap [iold] = leaf ;
    xold = x [leaf] ;
    while ( iold > 1 )
    {
        inew = iold/2 ;
        l = heap [inew] ;
        xnew = x [l] ;
        if ( xnew < xold )
        { /* swap new and old */
            heap [inew] = leaf ;
            heap [iold] = l ;
        }
        else return (n) ;
        iold = inew ;
    }
    return (n) ;
}

/* ========================================================================== */
/* === maxheapify =========================================================== */
/* ========================================================================== */

/* heapify starting at node p.  On input, the heap at node p satisfies the */
/* heap property, except for heap [p] itself.  On output, the whole heap */
/* satisfies the heap property. */

PRIVATE void ASLmaxheapify
(
    int   *heap, /* size n, containing indices into x */
    ASLFLOAT *x, /* not modified */
    int       p, /* start at node p in the heap */
    int       n  /* heap [1 ... n] is in use */
)
{
    int left, right, e, hleft, hright ;
    ASLFLOAT xe, xleft, xright ;

    e = heap [p] ;
    xe = x [e] ;

    while ( ASLTRUE )
    {
        left = p * 2 ;
        right = left + 1 ;

        if (right <= n)
        {
            hleft  = heap [left] ;
            hright = heap [right] ;
            xleft  = x [hleft] ;
            xright = x [hright] ;
            if (xleft > xright)
            {
                if (xe < xleft)
                {
                    heap [p] = hleft ;
                    p = left ;
                }
                else
                {
                    heap [p] = e ;
                    return ;
                }
            }
            else
            {
                if (xe < xright)
                {
                    heap [p] = hright ;
                    p = right ;
                }
                else
                {
                    heap [p] = e ;
                    return ;
                }
            }
        }
        else
        {
            if (left <= n)
            {
                hleft = heap [left] ;
                xleft = x [hleft] ;
                if (xe < xleft)
                {
                    heap [p] = hleft ;
                    p = left ;
                }
            }
            heap [p] = e ;
            return ;
        }
    }
}

/* ==========================================================================
   === ASLnapsack ===========================================================
   ==========================================================================
    Find x that minimizes ||x-y|| while satisfying lo <= x <= hi, a'x = b.
    The approach is to solve the dual problem obtained by introducing
    a multiplier lambda for the constraint a'x = b.  The dual function is 

    L (lambda) = min { .5||x-y||^2 + lambda (a'x - b): lo <= x <= hi }

    The dual function is concave. It is continuously differentiable.
    If mu denotes the maximizer of the dual function, then the solution
    of the primal problem is

    x = proj (y - mu*a) ,

    where proj (z) is the projection of z onto the set { x : lo <= x <= hi }.
    Thus we have

       proj (z)_i = lo_i if z_i <= lo_i,
                    hi_i if z_i >= hi_i,
                    z_i otherwise  .

    Note that for any lambda, the minimizing x in the dual function is

       x (lambda) = proj (y - lambda*a).

    The slope of the dual function is

      L'(lambda) = a'x(lambda) - b

    The solution technique is to start with an initial guess lambda for
    mu and search for a zero of L'. We have the following cases:

  1. L'(lambda) >= 0: mu >= lambda. If L' = 0, then done. Otherwise,
                      increase lambda using napup until slope vanishes

  2. L'(lambda) <= 0: mu <= lambda. If L' = 0, then done. Otherwise,
                      decrease lambda using napdown until slope vanishes

    The free set is those i for which lo_i < x_i (lambda) < hi_i.
    The active set is those i for which x_i = lo_i or x_i = hi_i.
    If we have a guess for the optimal active and free indices,
    then we can obtain a good guess for the starting lambda by
    setting the slope of the dual function to zero and solving for lambda.
    The guess for the active and free set is specified using the array B
    which has values:

        +1 => x_i = hi_i
        -1 => x_i = lo_i
         0 => lo_i < x_i < hi_i

    If there is no guess, then B is NULL.  Note that B is an INPUT array,
    it is not computed within the routine.

    The total time taken by napsack is O (n + h log n), where n is the size of
    y, h is the number of times an element of x (lambda) moves off the boundary
    into the free set plus the number of times elements move from the
    free set to the opposite boundary.  A heap is used to hold the entries
    in the boundary and in the free set.  If the slope vanishes at either
    the starting lambda, then no heap is constructed, and the time is
    just O (n).
   ========================================================================== */

PRIVATE ASLFLOAT ASLnapsack /* return the final lambda */
(
    ASLFLOAT     *x, /* holds y on input, and the solution x on output */
    ASLFLOAT lambda, /* starting guess for lambda */
    ASLFLOAT     *w, /* work array of size n */
    int        *iw1, /* integer work array of size n */
    int        *iw2, /* integer work array of size n */
    ASLcom     *Com  /* common variables */
)
{

    int a_is_1, bnd_is_scalar, j, n ;
    ASLFLOAT Clo, Chi, xj, atx, a2sum, aj, b, slope,
             *a, *lo, *hi ;

    lo = Com->lo ; /* lower bounds */
    hi = Com->hi ; /* upper bounds */
    a  = Com->a ;  /* a'x = b (linear constraint vector) */
    b  = Com->b ;  /* right side */
    n  = Com->n ;  /* problem dimension */
    bnd_is_scalar = Com->Parm->bnd_is_scalar ; /* TRUE means scalar bounds */
    if ( bnd_is_scalar )
    {
        Chi = *hi ;
        Clo = *lo ;
    }
    a_is_1 = Com->Parm->a_is_1 ;   /* TRUE means the a vector is 1 identically*/

    /* ---------------------------------------------------------------------- */
    /* compute the initial slope */
    /* ---------------------------------------------------------------------- */

    atx = ASLZERO ;
    a2sum = ASLZERO ;
    if ( a_is_1 )
    {
        if ( bnd_is_scalar )
        {
            for (j = 0; j < n; j++)
            {   
                xj = x [j] - lambda ;
                if      ( xj >= Chi ) atx += Chi ;
                else if ( xj <= Clo ) atx += Clo ;
                else                { atx += x [j] ; a2sum++ ; }
            }
        }
        else
        {
            for (j = 0; j < n; j++)
            {   
                xj = x [j] - lambda ;
                if      ( xj >= hi [j] ) atx += hi [j] ;
                else if ( xj <= lo [j] ) atx += lo [j] ;
                else                   { atx += x [j] ; a2sum++ ; }
            }
        }
    }
    else
    {
        if ( bnd_is_scalar )
        {
            for (j = 0; j < n; j++)
            {   
                aj = a [j] ;
                xj = x [j] - lambda*aj ;
                if      ( xj >= Chi ) atx += aj*Chi ;
                else if ( xj <= Clo ) atx += aj*Clo ;
                else                { atx += aj*x [j] ; a2sum += aj*aj ; }
            }
        }
        else
        {
            for (j = 0; j < n; j++)
            {   
                aj = a [j] ;
                xj = x [j] - lambda*aj ;
                if      ( xj >= hi [j] ) atx += aj*hi [j] ;
                else if ( xj <= lo [j] ) atx += aj*lo [j] ;
                else                   { atx += aj*x [j] ; a2sum += aj*aj ; }
            }
        }
    }
    slope = atx - lambda*a2sum - b ; /* slope of the dual function */
    if      ( slope > ASLZERO ) /* case 1, increase lambda */
    {
        lambda = ASLnapup (x, lambda, atx, a2sum, w, iw1-1, iw2-1, Com);
    }
    else if ( slope < ASLZERO ) /* case 2, decrease lambda */
    {
        lambda = ASLnapdown (x, lambda, atx, a2sum, w, iw1-1, iw2-1, Com) ;
    }
    /* if slope = 0, then done */

    /* ---------------------------------------------------------------------- */
    /* replace x by x (lambda) */
    /* ---------------------------------------------------------------------- */
    if ( a_is_1 )
    {
        if ( bnd_is_scalar )
        {
            for (j = 0; j < n; j++)
            {
                xj = x [j] - lambda ;
                if      ( xj < Clo ) xj = Clo ;
                else if ( xj > Chi ) xj = Chi ;
                x [j] = xj ;
            }
        }
        else
        {
            for (j = 0; j < n; j++)
            {
                xj = x [j] - lambda ;
                if      ( xj < lo [j] ) xj = lo [j] ;
                else if ( xj > hi [j] ) xj = hi [j] ;
                x [j] = xj ;
            }
        }
    }
    else
    {
        if ( bnd_is_scalar )
        {
            for (j = 0; j < n; j++)
            {
                xj = x [j] - lambda*a [j] ;
                if      ( xj < Clo ) xj = Clo ;
                else if ( xj > Chi ) xj = Chi ;
                x [j] = xj ;
            }
        }
        else
        {
            for (j = 0; j < n; j++)
            {
                xj = x [j] - lambda*a [j] ;
                if      ( xj < lo [j] ) xj = lo [j] ;
                else if ( xj > hi [j] ) xj = hi [j] ;
                x [j] = xj ;
            }
        }
    }

#ifdef DEBUG
    {
        asum = ASLZERO ;
        if ( a_is_1 ) for (j = 0; j < n; j++) asum += x [j] ;
        else          for (j = 0; j < n; j++) asum += a [j]*x [j] ;
        if ( fabs(asum-b) > 1.e-8 ) 
        {
            DPRINT (("final x infeasible, a'x = %e != b (%e)\n", asum, b)) ;
            ERROR ("stop") ;
        }
    }
#endif

    return (lambda) ;
}

/* ========================================================================== */
/* === ASLnapup ============================================================= */
/* ========================================================================== */

/* Find x that minimizes ||x-y|| while satisfying the constraints
   lo <= x <= hi, a'x = b.  The algorithm is described in the napsack comments.
   It is assumed that the starting guess lambda for the dual multiplier is <=
   the correct multiplier. Hence, lambda will be increased.  The slope of
   the dual function is a'x - b. We increase lambda until a'x reaches b.
   If a_i > 0, then as lambda increases, x_i decreases. If a_i < 0, then
   as lambda increases, x_i increases. */

PRIVATE ASLFLOAT ASLnapup /* return lambda */
(
    ASLFLOAT        *y,
    ASLFLOAT    lambda, /* initial guess for multiplier */
    ASLFLOAT       atx, /* a'x */
    ASLFLOAT     a2sum, /* sum_j a_j^2 */
    ASLFLOAT *breakpts, /* break points */
    int    *bound_heap, /* work array */
    int     *free_heap, /* work array */
    ASLcom        *Com  /* common variables */
    
)
{
    int a_is_1, bnd_is_scalar, j, k, maxsteps, n, n_bound, n_free ;
    ASLFLOAT aj, b, Clo, Chi, minbound, minfree, new_break, slope, t, xj,
             *a, *lo, *hi ;

    lo = Com->lo ; /* lower bounds */
    hi = Com->hi ; /* upper bounds */
    a  = Com->a ;  /* a'x = b (linear constraint vector) */
    b  = Com->b ;  /* right side */
    n  = Com->n ;  /* problem dimension */
    a_is_1 = Com->Parm->a_is_1 ;   /* TRUE means the a vector is 1 identically*/
    bnd_is_scalar = Com->Parm->bnd_is_scalar ; /* TRUE means scalar bounds */
    minbound = ASLINF ;
    minfree  = ASLINF ;

    /* -------------------------------------------------------------- */
    /* construct the heaps */
    /* -------------------------------------------------------------- */

    n_bound = 0 ;
    n_free = 0 ;
    if ( a_is_1 )
    {
        if ( bnd_is_scalar )
        {
            Clo = *lo ;
            Chi = *hi ;
            for (j = 0; j < n; j++)
            {   
                xj = y [j] - lambda ;
                if ( xj >= Chi )
                {   
                    n_bound++ ;
                    bound_heap [n_bound] = j ;
                    t = y [j] - Chi ;
                    if ( minbound > t ) minbound = t ;
                    breakpts [j] = t ;
                }
                else if ( xj > Clo )
                {   
                    n_free++ ;
                    free_heap [n_free] = j ;
                    t = y [j] - Clo ;
                    if ( minfree > t ) minfree = t ;
                    breakpts [j] = t ;
                }
            }
        }
        else
        {
            for (j = 0; j < n; j++)
            {   
                xj = y [j] - lambda ;
                if ( xj >= hi [j] )
                {   
                    n_bound++ ;
                    bound_heap [n_bound] = j ;
                    t = y [j] - hi [j] ;
                    if ( minbound > t ) minbound = t ;
                    breakpts [j] = t ;
                }
                else if ( xj > lo [j] )
                {   
                    n_free++ ;
                    free_heap [n_free] = j ;
                    t = y [j] - lo [j] ;
                    if ( minfree > t ) minfree = t ;
                    breakpts [j] = t ;
                }
            }
        }
    }
    else
    {
        if ( bnd_is_scalar )
        {
            Clo = *lo ;
            Chi = *hi ;
            for (j = 0 ; j < n ; j++)
            {
                aj = a [j] ;
                xj = y [j] - aj*lambda ;
                if ( aj > ASLZERO )
                {
                    if ( xj >= Chi )
                    {
                        n_bound++ ;
                        bound_heap [n_bound] = j ;
                        t = (y [j]-Chi)/aj ;
                        if ( minbound > t ) minbound = t ;
                        breakpts [j] = t ;
                    }
                    else if ( xj > Clo )
                    {
                        n_free++ ;
                        free_heap [n_free] = j ;
                        t = (y [j]-Clo)/aj ;
                        if ( minfree > t ) minfree = t ;
                        breakpts [j] = t ;
                    }
                }
                else if ( aj < ASLZERO )
                {
                    if ( xj <= Clo )
                    {
                        n_bound++ ;
                        bound_heap [n_bound] = j ;
                        t = (y [j]-Clo)/aj ;
                        if ( minbound > t ) minbound = t ;
                        breakpts [j] = t ;
                    }
                    else if ( xj < Chi )
                    {
                        n_free++ ;
                        free_heap [n_free] = j ;
                        t = (y [j]-Chi)/aj ;
                        if ( minfree > t ) minfree = t ;
                        breakpts [j] = t ;
                    }
                }
            }
        }
        else
        {
            for (j = 0 ; j < n ; j++)
            {
                aj = a [j] ;
                xj = y [j] - aj*lambda ;
                if ( aj > ASLZERO )
                {
                    if ( xj >= hi [j] )
                    {
                        n_bound++ ;
                        bound_heap [n_bound] = j ;
                        t = (y [j]-hi [j])/aj ;
                        if ( minbound > t ) minbound = t ;
                        breakpts [j] = t ;
                    }
                    else if ( xj > lo [j] )
                    {
                        n_free++ ;
                        free_heap [n_free] = j ;
                        t = (y [j]-lo [j])/aj ;
                        if ( minfree > t ) minfree = t ;
                        breakpts [j] = t ;
                    }
                }
                else if ( aj < ASLZERO )
                {
                    if ( xj <= lo [j] )
                    {
                        n_bound++ ;
                        bound_heap [n_bound] = j ;
                        t = (y [j]-lo [j])/aj ;
                        if ( minbound > t ) minbound = t ;
                        breakpts [j] = t ;
                    }
                    else if ( xj < hi [j] )
                    {
                        n_free++ ;
                        free_heap [n_free] = j ;
                        t = (y [j]-hi [j])/aj ;
                        if ( minfree > t ) minfree = t ;
                        breakpts [j] = t ;
                    }
                }
            }
        }
    }

    maxsteps = 2*n + 1;
    for (k = 1 ; k <= maxsteps ; k++)
    {
        new_break = ASLMIN (minfree, minbound) ;
        slope = atx - new_break*a2sum - b ;
        if ( (slope <= ASLZERO) || (new_break == ASLINF) ) /* done */
        {
            if ( a2sum != ASLZERO ) lambda = (atx-b)/a2sum ;
            return (lambda) ;
        }
        lambda = new_break ;

        if ( k == 1 )
        {
            ASLminheap_build (free_heap, breakpts, n_free) ;
            ASLminheap_build (bound_heap, breakpts, n_bound) ;
        }

        /* -------------------------------------------------------------- */
        /* update the heaps */
        /* -------------------------------------------------------------- */

        if ( n_free > 0 )
        {
            if ( a_is_1 )
            {
                while (breakpts [j = free_heap [1]] <= lambda)
                {
                    a2sum-- ;
                    if ( bnd_is_scalar ) atx +=  Clo - y [j] ;
                    else                 atx +=  lo [j] - y [j] ;
                    n_free = ASLminheap_delete (free_heap, breakpts, n_free) ;
                    if ( n_free == 0 ) break ;
                }
            }
            else
            {
                while (breakpts [j = free_heap [1]] <= lambda)
                {
                    aj = a [j] ;
                    a2sum -= aj*aj ;
                    if ( aj > ASLZERO )
                    {
                        if ( bnd_is_scalar ) atx += aj*(Clo-y [j]) ;
                        else                 atx += aj*(lo [j]-y [j]) ;
                    }
                    else
                    {
                        if ( bnd_is_scalar ) atx += aj*(Chi-y [j]) ;
                        else                 atx += aj*(hi [j]-y [j]) ;
                    }
                    n_free = ASLminheap_delete (free_heap, breakpts, n_free) ;
                    if ( n_free == 0 )
                    {
                        a2sum = ASLZERO ;
                        break ;
                    }
                }
            }
        }

        if ( n_bound > 0 )
        {
            if ( a_is_1 )
            {
                while (breakpts [j = bound_heap [1]] <= lambda)
                {
                    n_bound  = ASLminheap_delete (bound_heap, breakpts,n_bound);
                    a2sum++ ;
                    if ( bnd_is_scalar )
                    {
                        atx += y [j] - Chi ;
                        breakpts [j] = y [j] - Clo ;
                    }
                    else
                    {
                        atx += y [j] - hi [j] ;
                        breakpts [j] = y [j] - lo [j] ;
                    }
                    n_free = ASLminheap_add (j, free_heap, breakpts, n_free) ;
                    if ( n_bound == 0 ) break ;
                }
            }
            else
            {
                while (breakpts [j = bound_heap [1]] <= lambda)
                {
                    n_bound  = ASLminheap_delete(bound_heap, breakpts, n_bound);
                    aj = a [j] ;
                    a2sum += aj*aj ;
                    if ( aj > ASLZERO )
                    {
                        if ( bnd_is_scalar )
                        {
                            atx += aj*(y [j] - Chi) ;
                            breakpts [j] = (y [j]-Clo)/aj ;
                        }
                        else
                        {
                            atx += aj*(y [j] - hi [j]) ;
                            breakpts [j] = (y [j]-lo [j])/aj ;
                        }
                    }
                    else
                    {
                        if ( bnd_is_scalar )
                        {
                            atx += aj*(y [j] - Clo) ;
                            breakpts [j] = (y [j]-Chi)/aj ;
                        }
                        else
                        {
                            atx += aj*(y [j] - lo [j]) ;
                            breakpts [j] = (y [j]-hi [j])/aj ;
                        }
                    }
                    n_free = ASLminheap_add (j, free_heap, breakpts, n_free) ;
                    if ( n_bound == 0 ) break ;
                }
            }
        }

        /*------------------------------------------------------------------- */
        /* get the smallest entry in each heap */
        /*------------------------------------------------------------------- */

        if ( n_free > 0 )  minfree = breakpts [free_heap [1]] ;
        else               minfree = ASLINF ;
        if ( n_bound > 0 ) minbound = breakpts [bound_heap [1]] ;
        else               minbound = ASLINF ;
    }
    /* this should not happen */
    ERROR ("Cuttin' loose...") ;
    return (lambda) ;
}

/* ========================================================================== */
/* === ASLnapdown =========================================================== */
/* ========================================================================== */

/* Find x that minimizes ||x-y|| while satisfying the constraints
   lo <= x <= hi, a'x = b.  The algorithm is described in the napsack comments.
   It is assumed that the starting guess lambda for the dual multiplier is >=
   the correct multiplier. Hence, lambda will be decreased.  The slope of
   the dual function is a'x - b. We decrease lambda until a'x reaches b.
   If a_i > 0, then as lambda decreases, x_i increases. If a_i < 0, then
   as lambda decreases, x_i decreases. */

PRIVATE ASLFLOAT ASLnapdown /* return lambda */
(
    ASLFLOAT        *y,
    ASLFLOAT    lambda, /* initial guess for multiplier */
    ASLFLOAT       atx, /* a'x */
    ASLFLOAT     a2sum, /* sum_j a_j^2 */
    ASLFLOAT *breakpts, /* break points */
    int    *bound_heap, /* work array */
    int     *free_heap, /* work array */
    ASLcom        *Com  /* common variables */
    
)
{
    int a_is_1, bnd_is_scalar, j, k, maxsteps, n, n_bound, n_free ;
    ASLFLOAT aj, b, Clo, Chi, maxbound, maxfree, new_break, slope, t, xj,
            *a, *lo, *hi ;

    lo = Com->lo ; /* lower bounds */
    hi = Com->hi ; /* upper bounds */
    a  = Com->a ;  /* a'x = b (linear constraint vector) */
    b  = Com->b ;  /* right side */
    n  = Com->n ;  /* problem dimension */
    a_is_1 = Com->Parm->a_is_1 ;   /* TRUE means the a vector is 1 identically*/
    bnd_is_scalar = Com->Parm->bnd_is_scalar ; /* TRUE means scalar bounds */
    maxbound = -ASLINF ;
    maxfree  = -ASLINF ;

    /* -------------------------------------------------------------- */
    /* construct the heaps */
    /* -------------------------------------------------------------- */

    n_bound = 0 ;
    n_free = 0 ;
    if ( a_is_1 )
    {
        if ( bnd_is_scalar )
        {
            Clo = *lo ;
            Chi = *hi ;
            for (j = 0; j < n; j++)
            {   
                xj = y [j] - lambda ;
                if ( xj <= Clo )
                {   
                    n_bound++ ;
                    bound_heap [n_bound] = j ;
                    t = y [j] - Clo ;
                    if ( maxbound < t ) maxbound = t ;
                    breakpts [j] = t ;
                }
                else if ( xj < Chi )
                {   
                    n_free++ ;
                    free_heap [n_free] = j ;
                    t = y [j] - Chi ;
                    if ( maxfree < t ) maxfree = t ;
                    breakpts [j] = t ;
                }
            }
        }
        else
        {
            for (j = 0; j < n; j++)
            {   
                xj = y [j] - lambda ;
                if ( xj <= lo [j] )
                {   
                    n_bound++ ;
                    bound_heap [n_bound] = j ;
                    t = y [j] - lo [j] ;
                    if ( maxbound < t ) maxbound = t ;
                    breakpts [j] = t ;
                }
                else if ( xj < hi [j] )
                {   
                    n_free++ ;
                    free_heap [n_free] = j ;
                    t = y [j] - hi [j] ;
                    if ( maxfree < t ) maxfree = t ;
                    breakpts [j] = t ;
                }
            }
        }
    }
    else
    {
        if ( bnd_is_scalar )
        {
            Clo = *lo ;
            Chi = *hi ;
            for (j = 0 ; j < n ; j++)
            {
                aj = a [j] ;
                xj = y [j] - aj*lambda ;
                if ( aj > ASLZERO )
                {
                    if ( xj <= Clo )
                    {
                        n_bound++ ;
                        bound_heap [n_bound] = j ;
                        t = (y [j]-Clo)/aj ;
                        if ( maxbound < t ) maxbound = t ;
                        breakpts [j] = t ;
                    }
                    else if ( xj < Chi )
                    {
                        n_free++ ;
                        free_heap [n_free] = j ;
                        t = (y [j]-Chi)/aj ;
                        if ( maxfree < t ) maxfree = t ;
                        breakpts [j] = t ;
                    }
                }
                else if ( aj < ASLZERO )
                {
                    if ( xj >= Chi )
                    {
                        n_bound++ ;
                        bound_heap [n_bound] = j ;
                        t = (y [j]-Chi)/aj ;
                        if ( maxbound < t ) maxbound = t ;
                        breakpts [j] = t ;
                    }
                    else if ( xj > Clo )
                    {
                        n_free++ ;
                        free_heap [n_free] = j ;
                        t = (y [j]-Clo)/aj ;
                        if ( maxfree < t ) maxfree = t ;
                        breakpts [j] = t ;
                    }
                }
            }
        }
        else
        {
            for (j = 0 ; j < n ; j++)
            {
                aj = a [j] ;
                if ( aj > ASLZERO )
                {
                    xj = y [j] - aj*lambda ;
                    if ( xj <= lo [j] )
                    {
                        n_bound++ ;
                        bound_heap [n_bound] = j ;
                        t = (y [j]-lo [j])/aj ;
                        if ( maxbound < t ) maxbound = t ;
                        breakpts [j] = t ;
                    }
                    else if ( xj < hi [j] )
                    {
                        n_free++ ;
                        free_heap [n_free] = j ;
                        t = (y [j]-hi [j])/aj ;
                        if ( maxfree < t ) maxfree = t ;
                        breakpts [j] = t ;
                    }
                }
                else if ( aj < ASLZERO )
                {
                    xj = y [j] - aj*lambda ;
                    if ( xj >= hi [j] )
                    {
                        n_bound++ ;
                        bound_heap [n_bound] = j ;
                        t = (y [j]-hi [j])/aj ;
                        if ( maxbound < t ) maxbound = t ;
                        breakpts [j] = t ;
                    }
                    else if ( xj > lo [j] )
                    {
                        n_free++ ;
                        free_heap [n_free] = j ;
                        t = (y [j]-lo [j])/aj ;
                        if ( maxfree < t ) maxfree = t ;
                        breakpts [j] = t ;
                    }
                }
            }
        }
    }

    maxsteps = 2*n + 1;
    for (k = 1; k <= maxsteps; k++)
    {
        new_break = ASLMAX (maxfree, maxbound) ;
        slope = atx - new_break*a2sum - b ;
        if ( (slope >= ASLZERO) || (new_break == -ASLINF) ) /* done */
        {
            if ( a2sum != ASLZERO ) lambda = (atx-b)/a2sum ;
            return (lambda) ;
        }
        lambda = new_break ;

        if ( k == 1 )
        {
            ASLmaxheap_build (free_heap, breakpts, n_free) ;
            ASLmaxheap_build (bound_heap, breakpts, n_bound) ;
        }

        /* -------------------------------------------------------------- */
        /* update the heaps */
        /* -------------------------------------------------------------- */

        if ( n_free > 0 )
        {
            if ( a_is_1 )
            {
                while (breakpts [j = free_heap [1]] >= lambda)
                {
                    a2sum-- ;
                    if ( bnd_is_scalar ) atx +=  Chi - y [j] ;
                    else                 atx +=  hi [j] - y [j] ;
                    n_free = ASLmaxheap_delete (free_heap, breakpts, n_free) ;
                    if ( n_free == 0 ) break ;
                }
            }
            else
            {
                while (breakpts [j = free_heap [1]] >= lambda)
                {
                    aj = a [j] ;
                    a2sum -= aj*aj ;
                    if ( aj > ASLZERO )
                    {
                        if ( bnd_is_scalar ) atx += aj*(Chi-y [j]) ;
                        else                 atx += aj*(hi [j]-y [j]) ;
                    }
                    else
                    {
                        if ( bnd_is_scalar ) atx += aj*(Clo-y [j]) ;
                        else                 atx += aj*(lo [j]-y [j]) ;
                    }
                    n_free = ASLmaxheap_delete (free_heap, breakpts, n_free) ;
                    if ( n_free == 0 )
                    {
                        a2sum = ASLZERO ;
                        break ;
                    }
                }
            }
        }

        if ( n_bound > 0 )
        {
            if ( a_is_1 )
            {
                while (breakpts [j = bound_heap [1]] >= lambda)
                {
                    n_bound  = ASLmaxheap_delete (bound_heap, breakpts,n_bound);
                    a2sum++ ;
                    if ( bnd_is_scalar )
                    {
                        atx += y [j] - Clo ;
                        breakpts [j] = y [j] - Chi ;
                    }
                    else
                    {
                        atx += y [j] - lo [j] ;
                        breakpts [j] = y [j] - hi [j] ;
                    }
                    n_free = ASLmaxheap_add (free_heap, breakpts, j, n_free) ;
                    if ( n_bound == 0 ) break ;
                }
            }
            else
            {
                while (breakpts [j = bound_heap [1]] >= lambda)
                {
                    n_bound  = ASLmaxheap_delete(bound_heap, breakpts, n_bound);
                    aj = a [j] ;
                    a2sum += aj*aj ;
                    if ( aj > ASLZERO )
                    {
                        if ( bnd_is_scalar )
                        {
                            atx += aj*(y [j] - Clo) ;
                            breakpts [j] = (y [j]-Chi)/aj ;
                        }
                        else
                        {
                            atx += aj*(y [j] - lo [j]) ;
                            breakpts [j] = (y [j]-hi [j])/aj ;
                        }
                    }
                    else
                    {
                        if ( bnd_is_scalar )
                        {
                            atx += aj*(y [j] - Chi) ;
                            breakpts [j] = (y [j]-Clo)/aj ;
                        }
                        else
                        {
                            atx += aj*(y [j] - hi [j]) ;
                            breakpts [j] = (y [j]-lo [j])/aj ;
                        }
                    }
                    n_free = ASLmaxheap_add (free_heap, breakpts, j, n_free) ;
                    if ( n_bound == 0 ) break ;
                }
            }
        }

        /*------------------------------------------------------------------- */
        /* get the biggest entry in each heap */
        /*------------------------------------------------------------------- */

        if ( n_free > 0 )  maxfree = breakpts [free_heap [1]] ;
        else               maxfree = -ASLINF ;
        if ( n_bound > 0 ) maxbound = breakpts [bound_heap [1]] ;
        else               maxbound = -ASLINF ;
    }
    /* this should not happen */
    ERROR ("Cuttin' loose...") ;
    return (lambda) ;
}

/* ==========================================================================
   === ASLdefault ===========================================================
   ==========================================================================
       Set default parameter values.
   ========================================================================== */
PUBLIC void ASLdefault
(
    ASLparm *Parm /* pointer to parameter structure */
)
{
    Parm->epsmu0 = 1.e-9 ;  /* absolute error in mu computation */
    Parm->epsmu1 = 1.e-10 ; /* relative error in mu computation */
    Parm->lambda0 = 1.e-30 ;/* lower bound for lambda_k */
    Parm->lambda0Factor = 1.e-3 ; /* require lambda_k >= ||g||lambda0Factor */

    /* terminate Armijo line search when
       f(st) <= fR + st * delta * f'(0) where fR = mfax or fr */
    Parm->Armijo_delta = 1.0e-4 ;

    /* backtracking decay factor in the Armijo line search */
    Parm->Armijo_decay = 5.e-1 ;

    /* maximum number of times the Armijo line search will perform
       backtracking steps */
    Parm->max_backsteps = (int) 50 ;

    /* required decay factor (safe-guard factor) for width of bracketting
       interval when computing mu */
    Parm->gamma = .66 ;

    /* maximum number of iterations when computing mu */
    Parm->max_mu_its = 100 ;

    /* maximum number iterations in asl algorithm */
    Parm->maxits = ASLINT_INF ;

    /* fmax = max (f_{k-i}, i = 0, 1, ..., min (k, M-1) ) */
    Parm->M = 8 ;

    /* T => print parameters values
       F => do not display parameter values */
    Parm->PrintParms = ASLTRUE ;

    /* T => print final statistics
       F => no printout of statistics */
    Parm->PrintFinal = ASLTRUE ;

    /* Level 0 = no printing, ... , Level 2 = maximum printing */
    Parm->PrintLevel = 0 ;

    /* T => use approximate nonmonotone Armijo line search
       F => use ordinary nonmonotone Armijo line search, switch to
            approximate Armijo when |f_r-f| < AArmijoFac*|min (f_r, f_{max})|
       Set AArmijoFac = 0 and AArmijo = ASLFALSE to ensure ordinary Armijo */
    Parm->AArmijo = ASLFALSE ;

    Parm->AArmijoFac = 1.e-8 ;

    /* T => estimated error in function value = eps*|min (f_r, f_{max}) |
       F => estimated error in function value = eps */
    Parm->PertRule = ASLFALSE ;

    Parm->eps = 1.e-6 ;

    /* search for non nan function value by shrinking search interval
       at most nshrink times */
    Parm->nshrink = (int) 50 ;

    /* factor by which interval shrinks when searching for non nan value */
    Parm->nan_fac = 2.e-1 ; 

     /* update fr if fmin was not improved after L iterations */
    Parm->L = 3 ;

    /* update fr if initial stepsize was accepted in previous P iterations */
    Parm->P = 40 ;

    /* nominal CBB cycle length */
    Parm->nm = 4 ;

    /* absolute maximum number of iterations in a CBB cycle */
    Parm->MaxCycle = 6 ;

    /* update reference value fr if (fr-fmin)/(fc-fmin) > gamma1 */
    Parm->gamma1 = (ASLFLOAT) Parm->M / (ASLFLOAT) Parm->L ;

    /* update fr if (fr-f)/(fmax-f) > gamma2, np > P, and fmax > f */
    Parm->gamma2 = (ASLFLOAT) Parm->P / (ASLFLOAT) Parm->M ;

    /* use quadratic interpolation to compute Armijo step if it
       lies in the interval [armijo0*alpha, armijo1*alpha] */
    Parm->armijo0 = 1.e-10 ;
    Parm->armijo1 = 9.e-1 ; 

    /* set components of the search direction d to zero whenever the
       associated components of x are nearly active (within the cutoffs) */
    Parm->lo_cut = 0.e-12 ; /* g_j > 0 and x_j - lo_j < lo_cut => d_j = 0 */
    Parm->hi_cut = 0.e-12 ; /* g_j < 0 and hi_j - x_j < hi_cut => d_j = 0 */

    /* neglect components of y which satisfy yi >= y_s*big_yi where y_s <= 0
       is the smallest component of y */
    Parm->big_yi = 1.e-3 ;

    /* TRUE means that the a vector is identically 1 */
    Parm->a_is_1 = ASLFALSE ;

    /* TRUE means that the objective function is quadratic */
    Parm->QuadCost = ASLFALSE ;

    /* TRUE means that when the objective function is quadratic, the
       code should compute the starting gradient Ax + c and the
       starting objective function value .5x'Ax + c'x */
    Parm->QuadLazy = ASLTRUE ;

    /* TRUE means that the lower and upper bounds are scalar */
    Parm->bnd_is_scalar = ASLFALSE ;

    /* TRUE means that the algorithm works in a subspace */
    Parm->Subspace = ASLFALSE ;
    Parm->nsub = -1 ;  /* subspace size */
    Parm->fsub = .01 ; /* if nsub <= 0, subspace size = n*fsubspace */
}

/* ==========================================================================
   === ASLprint_parms =======================================================
   ==========================================================================
   print values in ASLparm structure
   ========================================================================== */
PRIVATE void ASLprint_parms
(
    ASLparm     *Parm, /* SSMparm structure to be printed */
    ASLFLOAT grad_tol, /* gradient tolerance */
    int             n  /* dimension */
)
{
    int nsub ;
    /* numerical parameters */
    printf ("print level (0 = none, 2 = maximum) .......... PrintLevel: %i\n",
             Parm->PrintLevel) ;
    printf ("stopping criterion ............................. grad_tol: %e\n",
             grad_tol) ;
    printf ("error tolerance for mu computation (B-A <= .. |g|) epsmu0: %e\n",
             Parm->epsmu0) ;
    printf ("bound for (B-A)/(|A|+|B|) in mu computation ...... epsmu1: %e\n",
             Parm->epsmu1) ;
    printf ("lower bound for lambda_k ........................ lambda0: %e\n",
             Parm->lambda0) ;
    printf ("require lambda_k >= ||g||*lambda0Factor ... lambda0Factor: %e\n",
             Parm->lambda0Factor) ;
    printf ("Armijo line search parameter ............... Armijo_delta: %e\n",
             Parm->Armijo_delta) ;
    printf ("Armijo decay factor .........................armijo_decay: %e\n",
             Parm->Armijo_decay) ;
    printf ("max number of Armijo backtracking steps ... max_backsteps: %i\n",
             Parm->max_backsteps) ;
    printf ("required decay factor in mu computation ........... gamma: %e\n",
             Parm->gamma) ;
    printf ("maximum number iterations when computing mu .. max_mu_its: %i\n",
             Parm->max_mu_its) ;
    printf ("maximum number of iterations ..................... maxits: %i\n",
             Parm->maxits) ;
    printf ("fmax = max (f_{k-i}, i = 0, 1, ..., min (k, M-1) )..... M: %i\n",
             Parm->M) ;
    printf ("cost change factor, approx Armijo transition,  AArmijoFac: %e\n",
             Parm->AArmijoFac) ;
    printf ("max tries to find non NAN function value ........ nshrink: %i\n",
             Parm->nshrink) ;
    printf ("perturbation parameter for function value............ eps: %e\n",
             Parm->eps) ;
    printf ("interval decay factor in NAN search ..............nan_fac: %e\n",
             Parm->nan_fac) ;
    printf ("update fr if fmin not improved after L iterations ..... L: %i\n",
             Parm->L) ;
    printf ("update fr if P previous initial stepsizes accepted .... P: %i\n",
             Parm->P) ;
    printf ("nominal CBB cycle length ............................. nm: %i\n",
             Parm->nm) ;
    printf ("absolute max number iteration in CBB cycle ..... MaxCycle: %i\n",
             Parm->MaxCycle) ;
    printf ("criterion for updating reference value fr......... gamma1: %e\n",
             Parm->gamma1) ;
    printf ("criterion for updating reference value fr......... gamma2: %e\n",
             Parm->gamma2) ;
    printf ("criterion for Q interpolation,  cbb line search,  armijo0: %e\n",
             Parm->armijo0) ;
    printf ("criterion for Q interpolation,  cbb line search,  armijo1: %e\n",
             Parm->armijo1) ;
    printf ("set d_j to 0 when x_j nearly active:\n") ;
    printf ("    g_j > 0 and x_j - lo_j < %e => d_j = 0\n", Parm->lo_cut) ;
    printf ("    g_j < 0 and hi_j - x_j < %e => d_j = 0\n", Parm->hi_cut) ;
    printf ("if y_i >= big_yi*(smallest y_i), ignore it ....... big_yi: %e\n",
             Parm->big_yi) ;

    /* logical parameters */
    printf ("\nLogical parameters:\n") ;
    if ( Parm->PrintFinal )
        printf ("    Print final cost and statistics\n") ;
    if ( Parm->PrintParms )
        printf ("    Print the parameter structure\n") ;
    else
        printf ("    Do not print parameter structure\n") ;
    if ( Parm->AArmijo)
    {
        printf ("    Approximate nonmonotone Armijo line search\n") ;
        if ( Parm->PertRule )
            printf ("    Error estimate for function value is eps*|fcomp|\n") ;
        else
            printf ("    Error estimate for function value is eps\n") ;
    }
    else
    {
        printf ("    Nonmonotone Armijo line search") ;
        if ( Parm->AArmijoFac > ASLZERO )
            printf (" ... switching to approx nonmonotone Armijo\n") ;
    }

    /* TRUE means that the a vector is identically 1 */
    if ( Parm->a_is_1 )
    {
        printf ("    The a vector is identically 1\n") ;
    }
    else
    {
        printf ("    The a vector is not assumed to be 1\n") ;
    }

    /* TRUE means that the objective function is quadratic*/
    if ( Parm->QuadCost )
    {
        printf ("    The objective function is quadratic\n") ;
        printf ("    User must provide routine to compute A*d\n") ;
        if ( Parm->QuadLazy )
        {
            printf ("    The g0 argument of ASL contains the "
                    "linear cost term c.\n") ;
            printf ("    The code should compute the starting gradient "
                    "Ax + c\nand the starting cost value .5x'Ax + c'x\n") ;
        }
        else
        {
            printf ("    The g0 argument of ASL contains the "
                    "start objective function gradient.\n") ;
            printf ("    The code returns the difference between the final "
                    "(optimal) objective function value\n") ;
            printf ("    and the starting value\n") ;
        }
    }
    else
    {
        printf ("    The objective function not declared quadratic\n") ;
    }
    /* TRUE means that the algorithm works in a subspace */
    if ( Parm->Subspace )
    {
        nsub = Parm->nsub ;
        if ( nsub <= 0 )
        {
            nsub = (int) n*Parm->fsub ;
            nsub = ASLMAX (2, nsub) ;
        }
        printf ("    The algorithm works in a subspace with dimension\n") ;
        printf ("    between %i and %i.\n", nsub, 2*nsub) ;
    }
    else
    {
        printf ("    The algorithm works in the full space\n") ;
    }
}

PRIVATE void error_handler
(
    char *message
)
{
    fprintf (stderr, "Error: %s\n", message) ;
    abort ( ) ;
}
